@@ -56,15 +56,15 @@ coot::molecule_t::set_base_colour_for_bonds(float r, float g, float b) {
 
 //! user-defined colour-index to colour
 void
-coot::molecule_t::set_user_defined_bond_colours(const std::map<unsigned int, std::array<float, 3> > &colour_map) {
+coot::molecule_t::set_user_defined_bond_colours(const std::map<unsigned int, std::array<float, 4> > &colour_map) {
 
    // Fill user_defined_bond_colours
 
-   std::map<unsigned int, std::array<float, 3> >::const_iterator it;
+   std::map<unsigned int, std::array<float, 4> >::const_iterator it;
    for (it=colour_map.begin(); it!=colour_map.end(); ++it) {
       unsigned int idx = it->first;
       const auto &col = it->second;
-      colour_holder ch(col[0], col[1], col[2]);
+      colour_holder ch(col[0], col[1], col[2], col[3]);
       // std::cout << "   " << idx << " " << ch << std::endl;
       user_defined_bond_colours[idx] = ch;
    }
@@ -295,6 +295,7 @@ coot::molecule_t::make_colour_by_chain_bonds(coot::protein_geometry *geom,
 
    bool draw_hydrogens_flag = draw_hydrogen_atoms_flag;
 
+   // Constructor M
    Bond_lines_container bonds(geom, no_bonds_to_these_atoms, draw_hydrogens_flag);
 
    bonds.add_rotamer_tables(tables_p);
@@ -1484,6 +1485,7 @@ coot::molecule_t::make_colour_table(bool dark_bg_flag) const {
 
             // was there a user-defined bond colour that superceeds this for this colour index?
             if (! user_defined_bond_colours.empty()) {
+
                std::map<unsigned int, colour_holder>::const_iterator it;
                it = user_defined_bond_colours.find(icol);
                if (it != user_defined_bond_colours.end()) {
@@ -1497,7 +1499,8 @@ coot::molecule_t::make_colour_table(bool dark_bg_flag) const {
                                << std::endl;
                }
             } else {
-               // std::cout << "debug:: Sad! user_defined_bond_colours was empty" << std::endl;
+               if (debug_colour_table)
+                  std::cout << "debug:: Sad! user_defined_bond_colours was empty" << std::endl;
             }
 
             if (debug_colour_table) { // debugging colours
@@ -1568,7 +1571,7 @@ std::vector<glm::vec4>
 coot::molecule_t::make_colour_table_for_goodsell_style(float colour_wheel_rotation_step, float saturation, float goodselliness) const {
 
    bool debug_colour_table = true;
-   float gcwrs = colour_wheel_rotation_step;
+   float gcwrs = colour_wheel_rotation_step / 360.0;
    std::vector<glm::vec4> colour_table;
    int icol_max = -1;
    for (int i=0; i<bonds_box.n_atom_centres_; i++) {
@@ -1584,16 +1587,22 @@ coot::molecule_t::make_colour_table_for_goodsell_style(float colour_wheel_rotati
          int icol = bonds_box.atom_centres_colour_[i];
          if (done_colours.find(icol) == done_colours.end()) {
             // c.f. molecules-class-info.cc lines 4000 or so
-            coot::colour_holder ch(0.8, 0.5, 0.6);
+            coot::colour_holder ch(0.8, 0.4, 0.6);
             int ic = icol - 100;
             bool is_C = !(ic%2);
             int chain_index =  ic/2;
-            float pastel_factor = 1.0/saturation;
-            ch.pastelize(pastel_factor);
+            if (is_C) {
+               float pastel_factor = 0.2;
+               ch.pastelize(pastel_factor);
+            }
             float rotation_angle = gcwrs * static_cast<float>(chain_index);
             ch.rotate_by(rotation_angle);
             if (is_C) ch.make_pale(goodselliness);
+            if (false)
+               std::cout << "colour table: " << icol << " rotation_angle " << rotation_angle << " "
+                         << ch.red << " " << ch.green << " " << ch.blue << std::endl;
             colour_table[icol] = glm::vec4(ch.red, ch.green, ch.blue, 1.0);
+            done_colours.insert(icol);
          }
       }
    }
@@ -1823,17 +1832,18 @@ coot::molecule_t::get_bonds_mesh(const std::string &mode, coot::protein_geometry
 
 
 generic_3d_lines_bonds_box_t
-coot::molecule_t::make_exportable_environment_bond_box(coot::residue_spec_t &spec, coot::protein_geometry &geom) const {
+coot::molecule_t::make_exportable_environment_bond_box(coot::residue_spec_t &spec, float max_dist, coot::protein_geometry &geom) const {
 
    // 20230327-PE when you make the libcootapi version, just export this function
    // and make a simple_mesh_t from the bonds_box_env.
    //
    auto make_bonds_box_env = [] (const coot::residue_spec_t &spec,
                                  atom_selection_container_t atom_sel,
+				 float max_dist,
                                  coot::protein_geometry &geom) {
       graphical_bonds_container bonds_box_env;
       float environment_min_distance = 0.0f;
-      float environment_max_distance = 3.5f;
+      float environment_max_distance = max_dist;
       bool draw_bonds_to_hydrogens_flag = true;
 
       mmdb::Residue *residue_p = coot::util::get_residue(spec, atom_sel.mol);
@@ -1868,7 +1878,7 @@ coot::molecule_t::make_exportable_environment_bond_box(coot::residue_spec_t &spe
       return bonds_box_env;
    };
 
-   graphical_bonds_container bonds_box_env = make_bonds_box_env(spec, atom_sel, geom);
+   graphical_bonds_container bonds_box_env = make_bonds_box_env(spec, atom_sel, max_dist, geom);
    return generic_3d_lines_bonds_box_t(bonds_box_env);
 
 }
@@ -1906,10 +1916,9 @@ coot::molecule_t::get_simple_molecule(int imol, mmdb::Residue *residue_p, bool d
       }
 
       for (int icol=0; icol<gbc.num_colours; icol++) {
-         glm::vec4 col = colour_table[icol];
          graphical_bonds_lines_list<graphics_line_t> &ll = gbc.bonds_[icol];
          for (int j=0; j<ll.num_lines; j++) {
-            bool thin = ll.thin_lines_flag; // do something with this?
+            // const bool &thin = ll.thin_lines_flag; // do something with this?
             simple::bond_t::bond_type_t bt = simple::bond_t::SINGLE_BOND;
             sm.add_bond(simple::bond_t(ll.pair_list[j].atom_index_1, ll.pair_list[j].atom_index_2, bt));
          }
@@ -1923,7 +1932,8 @@ coot::molecule_t::get_simple_molecule(int imol, mmdb::Residue *residue_p, bool d
    mmdb::Manager *new_mol = util::create_mmdbmanager_from_residue(residue_p);
    atom_selection_container_t atom_sel = make_asc(new_mol);
    Bond_lines_container bonds(geom_p, no_bonds_to_these_atoms, draw_hydrogen_atoms_flag);
-   bonds.do_colour_by_chain_bonds(atom_sel, false, imol, draw_hydrogen_atoms_flag, false, true, false, false);
+   bonds.do_colour_by_chain_bonds(atom_sel, false, imol, draw_hydrogen_atoms_flag,
+                                  false, true, false, false);
    bonds_box = bonds.make_graphical_bonds();
    sm = make_simple_molecule(bonds_box);
    atom_sel.clear_up();
