@@ -69,12 +69,11 @@ coot::molecule_t::is_EM_map() const {
 
    bool ret_is_em = false;
 
-   std::cout << "in coot::molecule::is_EM_map() A " << std::endl;
+   if (false)
+      std::cout << "in coot::molecule::is_EM_map() A " << std::endl;
 
    if (has_xmap()) {
-      std::cout << "in coot::molecule_t::is_EM_map() B " << is_em_map_cached_flag << std::endl;
       if (is_em_map_cached_flag == 1) { // -1 means unset
-         std::cout << "in coot::molecule_t::is_EM_map() C " << std::endl;
          ret_is_em = true;
       }
    }
@@ -95,6 +94,19 @@ coot::molecule_t::is_em_map_cached_state() {
    return is_em_map_cached_flag;
 }
 
+void
+coot::molecule_t::scale_map(float scale_factor) {
+
+   // can be multi-threaded?
+
+   if (has_xmap()) {
+      clipper::Xmap_base::Map_reference_index ix;
+      for (ix = xmap.first(); !ix.last(); ix.next() )
+	 xmap[ix] *= scale_factor;
+   }
+}
+
+
 void
 coot::molecule_t::set_map_is_difference_map(bool state) {
    xmap_is_diff_map = state;
@@ -315,10 +327,12 @@ void gensurf_and_add_vecs_threaded_workpackage(const clipper::Xmap<float> *xmap_
    try {
       CIsoSurface<float> my_isosurface;
 
+      bool use_vertex_gradients_for_map_normals_flag = false; // give user control
       coot::density_contour_triangles_container_t tri_con =
         my_isosurface.GenerateTriangles_from_Xmap(std::cref(*xmap_p),
                                                   contour_level, dy_radius, centre, isample_step,
-                                                  iream_start, n_reams, is_em_map);
+                                                  iream_start, n_reams, is_em_map,
+                                                  use_vertex_gradients_for_map_normals_flag);
 
       // we are about to put the triangles into draw_vectors, so get the lock to
       // do that, so that the threads don't try to change draw_vectors at the same time.
@@ -663,6 +677,18 @@ coot::molecule_t::difference_map_peaks(mmdb::Manager *mol, float n_rmsd) const {
 
 #include "coot-utils/xmap-stats.hh"
 
+// map functions, return -1.0 on not-a-map
+float
+coot::molecule_t::get_map_mean() const {
+
+   bool ignore_pseudo_zeros_for_map_stats = false; // set this to true for an EM map
+   bool ipz = ignore_pseudo_zeros_for_map_stats;
+   mean_and_variance<float> mv = map_density_distribution(xmap, 20, false, ipz);
+   float m = mv.mean;
+   return m;
+
+}
+
 // map functions, return -1.1 on not-a-map
 float
 coot::molecule_t::get_map_rmsd_approx() const {
@@ -1288,7 +1314,7 @@ coot::molecule_t::fit_to_map_by_random_jiggle(mmdb::PPAtom atom_selection,
                                                         mmr.second, chain_id,
                                                         mmr.first,
                                                         mmr.second, chain_id,
-                                                        COOT_LSQ_MAIN);
+                                                        lsq_t::MAIN);
                      matches.push_back(match);
                      mmdb::Manager *mol_1 = mol;
                      mmdb::Manager *mol_2 = atom_sel.mol;
@@ -1455,9 +1481,11 @@ coot::molecule_t::get_density_at_position(const clipper::Coord_orth &pos) const
 }
 
 texture_as_floats_t
-coot::molecule_t:: get_map_section_texture(int section_index, int axis) const {
+coot::molecule_t:: get_map_section_texture(int section_index, int axis,
+                                           float data_value_for_bottom,
+                                           float data_value_for_top) const {
 
-   texture_as_floats_t t(xmap, section_index, axis);
+   texture_as_floats_t t(xmap, section_index, axis, data_value_for_bottom, data_value_for_top);
    return t;
 }
 
@@ -1477,3 +1505,31 @@ coot::molecule_t::get_number_of_map_sections(int axis_id) const {
    }
    return n;
 }
+
+
+double
+coot::molecule_t::sum_density_for_atoms_in_residue(const std::string &cid,
+                                                   const std::vector<std::string> &atom_names,
+                                                   const clipper::Xmap<float> &xmap) const {
+
+   double v = 0.0;
+   mmdb::Residue *residue_p = cid_to_residue(cid);
+   if (residue_p) {
+      mmdb::Atom **residue_atoms = 0;
+      int n_residue_atoms = 0;
+      residue_p->GetAtomTable(residue_atoms, n_residue_atoms);
+      for (int iat=0; iat<n_residue_atoms; iat++) {
+         mmdb::Atom *at = residue_atoms[iat];
+         if (! at->isTer()) {
+            std::string atom_name = at->GetAtomName();
+            if (std::find(atom_names.begin(), atom_names.end(), atom_name) != atom_names.end()) {
+               clipper::Coord_orth pos = co(at);
+               float d = util::density_at_point(xmap, pos);
+               v += static_cast<double>(d);
+            }
+         }
+      }
+   }
+   return v;
+}
+
