@@ -38,6 +38,8 @@
 #include <MoleculesToTriangles/CXXClasses/VertexColorNormalPrimitive.h>
 #include <MoleculesToTriangles/CXXClasses/BallsPrimitive.h>
 
+#include "MoleculesToTriangles/CXXClasses/tubes.hh"
+
 #define GLM_ENABLE_EXPERIMENTAL
 #include <glm/gtx/string_cast.hpp>
 
@@ -45,6 +47,7 @@
 //! Add a colour rule: eg. ("//A", "red")
 void
 coot::molecule_t::add_colour_rule(const std::string &selection, const std::string &colour_name) {
+
    colour_rules.push_back(std::make_pair(selection, colour_name));
 }
 
@@ -142,7 +145,7 @@ coot::molecule_t::M2T_updateFloatParameter(const std::string &param_name, float
    bool done = false;
    for (unsigned int i=0; i<M2T_float_params.size(); i++) {
       if (param_name == M2T_float_params[i].first) {
-         M2T_float_params[i].first = value;
+         M2T_float_params[i].second = value;
          done = true;
          break;
       }
@@ -159,7 +162,7 @@ coot::molecule_t::M2T_updateIntParameter(const std::string &param_name, int valu
    bool done = false;
    for (unsigned int i=0; i<M2T_int_params.size(); i++) {
       if (param_name == M2T_int_params[i].first) {
-         M2T_int_params[i].first = value;
+         M2T_int_params[i].second = value;
          done = true;
          break;
       }
@@ -194,10 +197,12 @@ coot::molecule_t::print_M2T_IntParameters() const {
 #include "MoleculesToTriangles/CXXSurface/CXXSurface.h"
 #include "MoleculesToTriangles/CXXSurface/CXXCreator.h"
 
+
 coot::simple_mesh_t
 coot::molecule_t::get_molecular_representation_mesh(const std::string &atom_selection_str,
                                                     const std::string &colour_scheme,
-                                                    const std::string &style) const {
+                                                    const std::string &style,
+                                                    int secondaryStructureUsageFlag) const {
 
    bool debug = true;
 
@@ -261,10 +266,12 @@ coot::molecule_t::get_molecular_representation_mesh(const std::string &atom_sele
                   gv.normal[i] = vcn.normal[i];
                   gv.color[i]  = 0.0037f * vcn.color[i];
                }
+               gv.color[3] = 0.00392f * vcn.color[3];
                if ((gv.color[0] + gv.color[1] + gv.color[2]) > 10.0) {
-                  gv.color *= 0.00392; // /255
+                  gv.color *= 0.00392f; // /255
+                  if (gv.color[3] > 0.99)
+                     gv.color[3] = 1.0;
                }
-               gv.color[3] = 1.0;
             }
 
             auto indexArray = surface.getIndexArray();
@@ -362,92 +369,94 @@ coot::molecule_t::get_molecular_representation_mesh(const std::string &atom_sele
    };
 
    auto set_vertex_colour = [] (api::vnc_vertex &vertex, float potential) {
-      float pk = 10.0;
-      if (potential == 0.0) {
-         vertex.color = glm::vec4(0.6, 0.6, 0.6, 1.0);
-      } else {
-         if (potential < 0.0)
-            vertex.color = glm::vec4(0.6 - pk, 0.6 + pk * 0.5, 0.6 + pk * 0.5, 1.0);
-         else
-            vertex.color = glm::vec4(0.6 - pk * 0.5, 0.6 - pk * 0.5, 0.6 + pk, 1.0);
-      }
+       float pot_f = min(1.0,fabs(potential)*2.);
+       if (potential < 0.0){
+           vertex.color = glm::vec4(1.0, 1.0-pot_f, 1.0-pot_f, 1.0);
+       } else {
+           vertex.color = glm::vec4(1.0-pot_f, 1.0-pot_f, 1.0, 1.0);
+       }
    };
 
+   // ----------------- main line -------------------
+
    coot::simple_mesh_t mesh;
 
    if (false)
       std::cout << "get_molecular_representation_mesh() atom_selection: " << atom_selection_str
                 << " colour_scheme: " << colour_scheme << " style: " << style << std::endl;
 
-   try {
+   {
 
-      auto my_mol = std::make_shared<MyMolecule>(atom_sel.mol);
-      // auto chain_cs = ColorScheme::colorChainsScheme();
-      auto chain_cs = ColorScheme::colorChainsSchemeWithColourRules(colour_rules);
-      if (! colour_rules.empty())
-         chain_cs = ColorScheme::colorChainsSchemeWithColourRules(colour_rules);
-      auto ele_cs   = ColorScheme::colorByElementScheme();
-      auto ss_cs    = ColorScheme::colorBySecondaryScheme();
-      auto bf_cs    = ColorScheme::colorBFactorScheme();
-      auto this_cs  = chain_cs; // default
-      if (colour_scheme == "Chains")    this_cs = chain_cs;
-      if (colour_scheme == "Element")   this_cs = ele_cs;
-      if (colour_scheme == "BFactor")   this_cs = bf_cs;
-      if (colour_scheme == "Secondary") this_cs = ss_cs;
-      if (colour_scheme == "RampChains") {
-         mesh = ramp_chains(my_mol, atom_selection_str, style, M2T_float_params, M2T_int_params);
-      } else {
-
-         if (colour_scheme == "ByOwnPotential") {
-
-            std::shared_ptr<MolecularRepresentationInstance> molrepinst =
-               MolecularRepresentationInstance::create(my_mol, this_cs, atom_selection_str, style);
-            mesh = molecular_representation_instance_to_mesh(molrepinst, M2T_float_params, M2T_int_params);
-
-            //Instantiate an electrostatics map and cause it to calculate itself
-            CXXChargeTable theChargeTable;
-            CXXUtils::assignCharge(atom_sel.mol, atom_sel.SelectionHandle, &theChargeTable);
-            CXXCreator *theCreator = new CXXCreator(atom_sel.mol, atom_sel.SelectionHandle);
-            theCreator->calculate();
-            clipper::Cell cell;
-            clipper::NXmap<double> theClipperNXMap;
-            theClipperNXMap = theCreator->coerceToClipperMap(cell);
-
-            for (unsigned int i=0; i<mesh.vertices.size(); i++) {
-               clipper::Coord_orth orthogonals = glm_to_clipper(mesh.vertices[i].pos);
-               const clipper::Coord_map mapUnits(theClipperNXMap.coord_map(orthogonals));
-               float potential = theClipperNXMap.interp<clipper::Interp_cubic>( mapUnits );
-               // subSurfaceIter->setScalar(potentialHandle, i, potential);
-               // std::cout << "potential: " << potential << std::endl;  // 20240226-PE Quiet! (now that this test is run)
-               set_vertex_colour(mesh.vertices[i], potential); // change ref
-            }
-
-            delete theCreator;
+      try {
 
+         auto my_mol = std::make_shared<MyMolecule>(atom_sel.mol, secondaryStructureUsageFlag);
+         // auto chain_cs = ColorScheme::colorChainsScheme();
+         auto chain_cs = ColorScheme::colorChainsSchemeWithColourRules(colour_rules);
+         if (! colour_rules.empty())
+            chain_cs = ColorScheme::colorChainsSchemeWithColourRules(colour_rules);
+         auto ele_cs   = ColorScheme::colorByElementScheme();
+         auto ss_cs    = ColorScheme::colorBySecondaryScheme();
+         auto bf_cs    = ColorScheme::colorBFactorScheme();
+         auto this_cs  = chain_cs; // default
+         if (colour_scheme == "Chains")    this_cs = chain_cs;
+         if (colour_scheme == "Element")   this_cs = ele_cs;
+         if (colour_scheme == "BFactor")   this_cs = bf_cs;
+         if (colour_scheme == "Secondary") this_cs = ss_cs;
+         if (colour_scheme == "RampChains") {
+            mesh = ramp_chains(my_mol, atom_selection_str, style, M2T_float_params, M2T_int_params);
          } else {
 
-            std::shared_ptr<MolecularRepresentationInstance> molrepinst =
-               MolecularRepresentationInstance::create(my_mol, this_cs, atom_selection_str, style);
-            mesh = molecular_representation_instance_to_mesh(molrepinst, M2T_float_params, M2T_int_params);
+            if (colour_scheme == "ByOwnPotential") {
+
+               std::shared_ptr<MolecularRepresentationInstance> molrepinst =
+                  MolecularRepresentationInstance::create(my_mol, this_cs, atom_selection_str, style);
+               mesh = molecular_representation_instance_to_mesh(molrepinst, M2T_float_params, M2T_int_params);
+
+               //Instantiate an electrostatics map and cause it to calculate itself
+               CXXChargeTable theChargeTable;
+               CXXUtils::assignCharge(atom_sel.mol, atom_sel.SelectionHandle, &theChargeTable);
+               CXXCreator *theCreator = new CXXCreator(atom_sel.mol, atom_sel.SelectionHandle);
+               theCreator->calculate();
+               clipper::Cell cell;
+               clipper::NXmap<double> theClipperNXMap;
+               theClipperNXMap = theCreator->coerceToClipperMap(cell);
 
-            if (false) {
                for (unsigned int i=0; i<mesh.vertices.size(); i++) {
-                  const auto &vertex = mesh.vertices[i];
-                  std::cout << i << " " << glm::to_string(vertex.pos) << " " << glm::to_string(vertex.color) << std::endl;
+                  clipper::Coord_orth orthogonals = glm_to_clipper(mesh.vertices[i].pos);
+                  const clipper::Coord_map mapUnits(theClipperNXMap.coord_map(orthogonals));
+                  float potential = theClipperNXMap.interp<clipper::Interp_cubic>( mapUnits );
+                  // subSurfaceIter->setScalar(potentialHandle, i, potential);
+                  // std::cout << "potential: " << potential << std::endl;  // 20240226-PE Quiet! (now that this test is run)
+                  set_vertex_colour(mesh.vertices[i], potential); // change ref
+               }
+
+               delete theCreator;
+
+            } else {
+
+               std::shared_ptr<MolecularRepresentationInstance> molrepinst =
+                  MolecularRepresentationInstance::create(my_mol, this_cs, atom_selection_str, style);
+               mesh = molecular_representation_instance_to_mesh(molrepinst, M2T_float_params, M2T_int_params);
+
+               if (false) {
+                  for (unsigned int i=0; i<mesh.vertices.size(); i++) {
+                     const auto &vertex = mesh.vertices[i];
+                     std::cout << i << " " << glm::to_string(vertex.pos) << " " << glm::to_string(vertex.color) << std::endl;
+                  }
                }
             }
          }
+         mesh.fill_colour_map(); // for blendering
+      }
+      catch (const std::out_of_range &oor) {
+         std::cout << "ERROR:: out of range in get_molecular_representation_mesh() " << oor.what() << std::endl;
+      }
+      catch (const std::runtime_error &rte) {
+         std::cout << "ERROR:: runtime error in get_molecular_representation_mesh() " << rte.what() << std::endl;
+      }
+      catch (...) {
+         std::cout << "ERROR:: unknown exception in get_molecular_representation_mesh()! " << std::endl;
       }
-      mesh.fill_colour_map(); // for blendering
-   }
-   catch (const std::out_of_range &oor) {
-      std::cout << "ERROR:: out of range in get_molecular_representation_mesh() " << oor.what() << std::endl;
-   }
-   catch (const std::runtime_error &rte) {
-      std::cout << "ERROR:: runtime error in get_molecular_representation_mesh() " << rte.what() << std::endl;
-   }
-   catch (...) {
-      std::cout << "ERROR:: unknown exception in get_molecular_representation_mesh()! " << std::endl;
    }
 
    return mesh;
