@@ -1,10 +1,12 @@
 
 #include <clipper/ccp4/ccp4_map_io.h> // debugging mapout
 
+#include "utils/base64-encode-decode.hh"
 #include "ligand/wligand.hh"
 
 #include "molecules-container.hh"
 
+
 // Give this ex-lambda function a home?
 std::string get_first_residue_name(mmdb::Manager *mol) {
 
@@ -42,6 +44,8 @@ molecules_container_t::get_rdkit_mol(const std::string &residue_name, int imol_e
       if (r_p.first) {
          const auto &restraints = r_p.second;
          m = coot::rdkit_mol(restraints);
+         std::string prop_string = "ligand-from-dictionary-" + residue_name + "-" + std::to_string(imol_enc);
+         m.setProp("moorhen-id", prop_string);
       }
    }
    catch (const std::runtime_error &rte) {
@@ -52,6 +56,44 @@ molecules_container_t::get_rdkit_mol(const std::string &residue_name, int imol_e
 #endif
 
 
+#ifdef MAKE_ENHANCED_LIGAND_TOOLS
+
+// Prevents preprocessor substitution of `VERSION` in `MolPickler.h`
+#ifndef RD_MOLPICKLE_H
+
+#ifdef VERSION
+#define __COOT_VERSION_VALUE VERSION
+#undef VERSION
+#endif
+
+#include <GraphMol/MolPickler.h>
+
+#ifdef __COOT_VERSION_VALUE
+#define VERSION __COOT_VERSION_VALUE
+#undef __COOT_VERSION_VALUE
+#endif
+
+#endif //RD_MOLPICKLE_H
+
+
+std::string
+molecules_container_t::get_rdkit_mol_pickle_base64(const std::string &residue_name, int imol_enc) {
+
+   RDKIT_GRAPHMOL_EXPORT RDKit::MolPickler mp;
+   std::string pickle_string;
+   RDKit::RWMol mol = get_rdkit_mol(residue_name, imol_enc);
+   if (mol.getNumAtoms() > 0) {
+      mp.pickleMol(mol, pickle_string);
+      return moorhen_base64::base64_encode((const unsigned char*)pickle_string.c_str(), pickle_string.size());
+      // std::ofstream f("test-mol.pickle");
+      // f << pickle_string;
+      // f.close();
+   }
+   return pickle_string;
+}
+#endif
+
+
 //! Ligand Fit
 //! @return a vector or the best fitting ligands to this blob.
 //! I am not yet clear what extra cut-offs and flags need to be added here.
@@ -454,19 +496,24 @@ molecules_container_t::get_eigenvalues(int imol, const std::string &chain_id, in
 #ifdef MAKE_ENHANCED_LIGAND_TOOLS
 #include "lidia-core/rdkit-interface.hh"
 #include <GraphMol/MolDraw2D/MolDraw2DSVG.h>
-#endif
-
 #include "lidia-core/svg-molecule.hh"
 #include "svg-store-key.hh"
+#endif
 
 //! This is a ligand function, not really a ligand-fitting function.
 //!
 std::string
 molecules_container_t::get_svg_for_residue_type(int imol, const std::string &comp_id,
-                                                bool use_rdkit_svg, bool dark_bg_flag) {
+                                                bool use_rdkit_svg,
+                                                const std::string &background_type) {
 
    std::string s = "Needs-to-be-compiled-with-the-RDKit"; // gets changed to the correct string (hopefully)
 
+   bool dark_bg_flag = false;
+   // "dark-mode" representation
+   if (background_type == "light-bonds/transparent-bg") dark_bg_flag = true;
+   if (background_type == "light-bonds/opaque-bg")      dark_bg_flag = true;
+
 #ifdef MAKE_ENHANCED_LIGAND_TOOLS
 
    svg_store_key_t key(imol, comp_id);
@@ -479,6 +526,7 @@ molecules_container_t::get_svg_for_residue_type(int imol, const std::string &com
 
       std::pair<bool, coot::dictionary_residue_restraints_t> mr = geom.get_monomer_restraints(comp_id, imol);
       if (mr.first) {
+
          try {
             const coot::dictionary_residue_restraints_t &restraints = mr.second;
             std::pair<int, RDKit::RWMol> mol_pair = coot::rdkit_mol_with_2d_depiction(restraints);
@@ -504,7 +552,11 @@ molecules_container_t::get_svg_for_residue_type(int imol, const std::string &com
                } else {
                   svg_molecule_t svg;
                   svg.import_rdkit_mol(&mol, iconf);
-                  s = svg.render_to_svg_string(dark_bg_flag);
+                  double sf = 400.0;
+                  bool add_bg = true;
+                  if (background_type == "light-bonds/transparent-bg") add_bg = false;
+                  if (background_type == "dark-bonds/transparent-bg")  add_bg = false;
+                  s = svg.render_to_svg_string(sf, dark_bg_flag, add_bg);
                   ligand_svg_store[key] = s;
                }
             } else {
@@ -534,8 +586,11 @@ molecules_container_t::get_svg_for_residue_type(int imol, const std::string &com
                   RDKit::WedgeMolBonds(rdkit_mol, &conf);
                   svg_molecule_t svg;
                   svg.import_rdkit_mol(&rdkit_mol, conformer_id);
-                  dark_bg_flag = false;
-                  s = svg.render_to_svg_string(dark_bg_flag);
+                  double sf = 400.0;
+                  bool add_background_rect = true;
+                  if (background_type == "light-bonds/transparent-bg") add_background_rect = false;
+                  if (background_type == "dark-bonds/transparent-bg")  add_background_rect = false;
+                  s = svg.render_to_svg_string(sf, dark_bg_flag, add_background_rect);
                }
                ligand_svg_store[key] = s;
             }
@@ -543,6 +598,12 @@ molecules_container_t::get_svg_for_residue_type(int imol, const std::string &com
          catch (const Invar::Invariant &e) {
             std::cout << "error " << e.what() << std::endl;
          }
+         catch (const std::runtime_error &e) {
+            std::cout << "error " << e.what() << std::endl;
+         }
+         catch (...) {
+            std::cout << "error something" << std::endl;
+         }
 
       } else {
          s = std::string("No dictionary for ") + comp_id;
