@@ -3,6 +3,34 @@
 #include "coot-utils/atom-selection-container.hh"
 #include "molecules-container.hh"
 
+//! Copy the molecule
+//!
+//! @param imol the specified molecule
+//! @return the new molecule number
+int
+molecules_container_t::copy_molecule(int imol) {
+
+   int imol_new = -1;
+   if (is_valid_model_molecule(imol)) {
+      imol_new = molecules.size();
+      mmdb::Manager *mol = coot::util::copy_molecule(molecules[imol].atom_sel.mol);
+      atom_selection_container_t asc = make_asc(mol);
+      std::string new_name = "copy-of-molecule-" + std::to_string(imol);
+      molecules.push_back(coot::molecule_t(asc, imol_new, new_name));
+   }
+
+   if (is_valid_map_molecule(imol)) {
+      imol_new = molecules.size();
+      std::string new_name = "copy-of-molecule-" + std::to_string(imol);
+      const clipper::Xmap<float> &xmap = molecules[imol].xmap;
+      bool is_em = molecules[imol].is_EM_map();
+      molecules.push_back(coot::molecule_t(new_name, imol_new, xmap, is_em));
+   }
+   return imol_new;
+}
+
+
+
 //! return the new molecule number (or -1 on no atoms selected)
 int
 molecules_container_t::copy_fragment_using_cid(int imol, const std::string &multi_cids) {
@@ -19,12 +47,18 @@ molecules_container_t::copy_fragment_using_cid(int imol, const std::string &mult
       mmdb::Manager *new_manager = coot::util::create_mmdbmanager_from_atom_selection(mol, selHnd);
       if (new_manager) {
          int transfer_atom_index_handle = new_manager->GetUDDHandle(mmdb::UDR_ATOM, "transfer atom index");
+
          // std::cout << "..... transfer_atom_index_handle A " << transfer_atom_index_handle << std::endl;
+
          imol_new = molecules.size();
          atom_selection_container_t asc = make_asc(new_manager);
          asc.UDDOldAtomIndexHandle = transfer_atom_index_handle;
          std::string new_name = "copy-fragment-from-molecule-" + std::to_string(imol);
          molecules.push_back(coot::molecule_t(asc, imol_new, new_name));
+         if (false)
+            std::cout << "debug:: in mc::copy_fragment_using_cid(): the UDDOldAtomIndexHandle for molecule " << imol_new
+                      << " is " << molecules[imol_new].atom_sel.UDDOldAtomIndexHandle << std::endl;
+
       }
       mol->DeleteSelection(selHnd);
    } else {
@@ -59,7 +93,6 @@ molecules_container_t::copy_fragment_for_refinement_using_cid(int imol, const st
       if (new_manager) {
 
          int transfer_atom_index_handle = new_manager->GetUDDHandle(mmdb::UDR_ATOM, "transfer atom index");
-         std::cout << ".......................... transfer_atom_index_handle B " << transfer_atom_index_handle << std::endl;
          // create a new molecule
          imol_new = molecules.size();
          atom_selection_container_t asc = make_asc(new_manager);
@@ -292,7 +325,7 @@ molecules_container_t::add_compound(int imol, const std::string &tlc, int imol_d
                   std::vector<mmdb::Manager *> mols = { mol };
                   auto merge_results = merge_molecules(imol, mols);
 
-                  std::cout << "llllllllllllllllllllllllll in add_compound() with merge_results size "
+                  std::cout << "debug:: in add_compound() with merge_results size "
                             << merge_results.first << " "
                             << merge_results.second.size() << std::endl;
 
@@ -420,7 +453,13 @@ molecules_container_t::minimize_energy(int imol, const std::string &atom_selecti
                                         refinement_is_quiet, &geom);
       std::string mode = "COLOUR-BY-CHAIN-AND-DICTIONARY";
       bool draw_hydrogen_atoms_flag = true; // use data member as we do for draw_missing_residue_loops_flag?
-      im = molecules[imol].get_bonds_mesh_instanced(mode, &geom, true, 0.12, 1.4, 1,
+      unsigned int smoothness_factor = 1;
+      bool show_atoms_as_aniso_flag = false;
+      bool show_aniso_atoms_as_ortep = false;
+      im = molecules[imol].get_bonds_mesh_instanced(mode, &geom, true, 0.12, 1.4,
+                                                    show_atoms_as_aniso_flag,
+                                                    show_aniso_atoms_as_ortep,
+                                                    smoothness_factor,
                                                     draw_hydrogen_atoms_flag, draw_missing_residue_loops_flag);
 
    } else {
@@ -432,6 +471,39 @@ molecules_container_t::minimize_energy(int imol, const std::string &atom_selecti
 
 }
 
+//! @param imol is the model molecule index
+//! @param atom_selection_cid is the selection CID e.g. "//A/15" (residue 15 of chain A)
+//! @param n_cycles is the number of refinement cycles. If you pass n_cycles = 100 (or some such) then you can
+//!         get the mesh for the partially optimized ligand/residues
+//! @param do_rama_plot_restraints is the flag for the usage of Ramachandran plot restraints
+//! @param rama_plot_weight is the flag to set the Ramachandran plot restraints weight
+//! @param do_torsion_restraints is the flag for the usage of torsion restraints
+//! @param torsion_weight is the flag to set the torsion restraints weight
+//! @param refinement_is_quiet is used to reduce the amount of diagnostic text written to the output
+//!
+//! @return the function value at termination
+float
+molecules_container_t::minimize(int imol, const std::string &atom_selection_cid,
+                                int n_cycles,
+                                bool do_rama_plot_restraints, float rama_plot_weight,
+                                bool do_torsion_restraints, float torsion_weight, bool refinement_is_quiet) {
+
+   int status = 0;
+   coot::instanced_mesh_t im;
+   if (is_valid_model_molecule(imol)) {
+      status = molecules[imol].minimize(atom_selection_cid, n_cycles,
+                                        do_rama_plot_restraints, rama_plot_weight,
+                                        do_torsion_restraints, torsion_weight,
+                                        refinement_is_quiet, &geom);
+
+   } else {
+      std::cout << "WARNING:: " << __FUNCTION__ << "(): not a valid model molecule " << imol << std::endl;
+   }
+
+   return status; // this is the wrong return value.
+}
+
+
 
 coot::graph_match_info_t
 molecules_container_t::overlap_ligands_internal(int imol_ligand, int imol_ref, const std::string &chain_id_ref,
@@ -618,3 +690,175 @@ molecules_container_t::match_ligand_torsions_and_position(int imol_ligand, int i
    }
    return status;
 }
+
+void 
+molecules_container_t::associate_sequence(int imol, const std::string &chain_id, const std::string &sequence) {
+
+   if (is_valid_model_molecule(imol)) { // but also maps?
+      molecules[imol].associate_sequence_with_molecule(chain_id, sequence);
+   }
+
+}
+
+void
+molecules_container_t::assign_sequence(int imol_model, int imol_map) {
+
+   if (is_valid_model_molecule(imol_model)) {
+       if (is_valid_map_molecule(imol_map)) {
+           const clipper::Xmap<float> &xmap = molecules[imol_map].xmap;
+           molecules[imol_model].assign_sequence(xmap, geom);
+       }
+   }
+}
+
+
+//! Get the conformers that can be generated by variation around rotatable bonds as described in the dictionary.
+//! Torsions that are marked as "const" are excluded from the variation, as are pyranose ring torsions
+//! and torsions that rotate hydrogen atoms.
+//! @return a vector of indices of the new molecules
+std::vector<int>
+molecules_container_t::get_dictionary_conformers(const std::string &comp_id, int imol_enc, bool remove_internal_clash_conformers) {
+
+   std::vector<int> mol_indices;
+   std::pair<bool, coot::dictionary_residue_restraints_t> r = geom.get_monomer_restraints(comp_id, imol_enc);
+   if (r.first) {
+      bool delete_clash_confs = true;
+      std::vector<mmdb::Residue *> confs = coot::util::get_dictionary_conformers(r.second, delete_clash_confs);
+      for (unsigned int i=0; i<confs.size(); i++) {
+         mmdb::Residue *res = confs[i];
+         mmdb::Manager *mol = coot::util::create_mmdbmanager_from_residue(res);
+         std::string name = comp_id + "-conf-" + std::to_string(i);
+         atom_selection_container_t asc = make_asc(mol);
+         int imol_no = molecules.size();
+         molecules.push_back(coot::molecule_t(asc, imol_no, name));
+         mol_indices.push_back(imol_no);
+      }
+      for (unsigned int i=0; i<confs.size(); i++)
+         delete confs[i];
+   }
+   return mol_indices;
+}
+
+//! replace a residue
+//!
+//! Change the type of a residue (for example, "TYR" to "PTY").
+//! The algorithm will superpose the mainchain CA, C and N and try to set matching torsion
+//! to the angle that they were in the reference structure.
+void
+molecules_container_t::replace_residue(int imol, const std::string &residue_cid,
+                                       const std::string &new_residue_type, int imol_enc) {
+
+   int status = 0;
+   if (is_valid_model_molecule(imol)) {
+      status = molecules[imol].replace_residue(residue_cid, new_residue_type, imol_enc, geom);
+   } else {
+      std::cout << "WARNING:: " << __FUNCTION__ << "(): not a valid model molecule " << imol << std::endl;
+   }
+   // return status
+}
+
+
+//! Rotate atoms around torsion
+//!
+//! the bond is presumed to be between atom-2 and atom-3. Atom-1 and atom-4 are
+//! used to define the absolute torsion angle.
+//!
+//! @return status 1 if successful, 0 if not.
+int
+molecules_container_t::rotate_around_bond(int imol, const std::string &residue_cid,
+                                          const std::string &atom_name_1,
+                                          const std::string &atom_name_2,
+                                          const std::string &atom_name_3,
+                                          const std::string &atom_name_4,
+                                          double torsion_angle) {
+
+   std::string alt_conf = "";
+   int status = 0;
+   if (is_valid_model_molecule(imol)) {
+      coot::atom_name_quad quad(atom_name_1, atom_name_2, atom_name_3, atom_name_4);
+      status = molecules[imol].rotate_around_bond(residue_cid, alt_conf, quad, torsion_angle, geom);
+   } else {
+      std::cout << "WARNING:: " << __FUNCTION__ << "(): not a valid model molecule " << imol
+                << std::endl;
+   }
+   return status;
+}
+
+
+
+
+//! change the alt confs
+//!
+//! @param change_mode is either "residue", "side-chain" or a comma-separated atom-name
+//! pairs (e.g "N,CA") - you can (of course) specify just one atom: "N".
+// @return the success status (1 is done, 0 means failed to do)
+int
+molecules_container_t::change_alt_locs(int imol, const std::string &cid, const std::string &change_mode) {
+
+   int status = 0;
+   if (is_valid_model_molecule(imol)) {
+      status = molecules[imol].change_alt_locs(cid, change_mode);
+   } else {
+      std::cout << "WARNING:: " << __FUNCTION__ << "(): not a valid model molecule " << imol << std::endl;
+   }
+   return status;
+}
+
+
+coot::instanced_mesh_t
+molecules_container_t::get_HOLE(int imol,
+                                float start_pos_x, float start_pos_y, float start_pos_z,
+                                float end_pos_x, float end_pos_y, float end_pos_z) const {
+
+   coot::instanced_mesh_t m;
+   if (is_valid_model_molecule(imol)) {
+      clipper::Coord_orth start_pos(start_pos_x, start_pos_y, start_pos_z);
+      clipper::Coord_orth end_pos(end_pos_x, end_pos_y, end_pos_z);
+      m = molecules[imol].get_HOLE(start_pos, end_pos, geom);
+   } else {
+      std::cout << "WARNING:: " << __FUNCTION__ << "(): not a valid model molecule " << imol << std::endl;
+   }
+   return m;
+
+}
+
+//! Get SVG for 2d ligand environment view (FLEV)
+//!
+//! The caller should make sure that the dictionary for the ligand has been loaded - this
+//! function won't do that. It will add hydrogen atoms if needed.
+//!
+//! @param imol is the model molecule index
+//! @param residue_cid is the cid for the residue
+std::string
+molecules_container_t::get_svg_for_2d_ligand_environment_view(int imol,
+                                                              const std::string &residue_cid,
+                                                              bool add_key) {
+
+   float radius = 4.2; // pass this
+   std::string s;
+   if (is_valid_model_molecule(imol)) {
+      s = molecules[imol].get_svg_for_2d_ligand_environment_view(residue_cid, &geom, add_key);
+   } else {
+      std::cout << "WARNING:: " << __FUNCTION__ << "(): not a valid model molecule " << imol << std::endl;
+   }
+   return s;
+}
+
+
+
+//! delete all carbohydrate
+//!
+//! @param imol is the model molecule index
+//!
+//! @return true on successful deletion, return false on no deletion.
+bool
+molecules_container_t::delete_all_carbohydrate(int imol) {
+
+   bool status = false;
+   if (is_valid_model_molecule(imol)) {
+      status = molecules[imol].delete_all_carbohydrate();
+   } else {
+      std::cout << "WARNING:: " << __FUNCTION__ << "(): not a valid model molecule " << imol << std::endl;
+   }
+   return status;
+}
