@@ -1,3 +1,4 @@
+
 #include <nanobind/nanobind.h>
 #include <nanobind/stl/string.h>
 #include <nanobind/stl/pair.h>
@@ -6,14 +7,16 @@
 #include <nanobind/ndarray.h>
 #include <nanobind/operators.h>
 
-#include "molecules-container.hh"
-#include "mini-mol/mini-mol-utils.hh"
-
 #include "clipper/core/ramachandran.h"
 #include "clipper/clipper-ccp4.h"
 
-#include "coot-utils/g_triangle.hh"
 #include "coords/mmdb-crystal.h"
+#include "coot-utils/acedrg-types-for-residue.hh"
+#include "coot-utils/g_triangle.hh"
+#include "mini-mol/mini-mol-utils.hh"
+
+#undef MAKE_ENHANCED_LIGAND_TOOLS
+#include "molecules-container.hh"
 
 namespace nb = nanobind;
 
@@ -58,7 +61,7 @@ class molecules_container_js : public molecules_container_t {
         }
 };
 
-NB_MODULE(chapi, m) {
+NB_MODULE(coot_headless_api, m) {
     nb::class_<clipper::Coord_orth>(m,"Coord_orth")
     .def(nb::init<const clipper::ftype&, const clipper::ftype&, const clipper::ftype&>())
     .def("x", &clipper::Coord_orth::x)
@@ -189,37 +192,47 @@ NB_MODULE(chapi, m) {
     nb::class_<molecules_container_t>(m,"molecules_container_t")
     .def(nb::init<bool>())
     .def("M2T_updateFloatParameter",&molecules_container_t::M2T_updateFloatParameter)
+    .def("M2T_updateIntParameter",&molecules_container_t::M2T_updateIntParameter)
     .def("SSM_superpose",&molecules_container_t::SSM_superpose)
     .def("add_alternative_conformation",&molecules_container_t::add_alternative_conformation)
     .def("add_colour_rule",&molecules_container_t::add_colour_rule)
     .def("add_colour_rules_multi",&molecules_container_t::add_colour_rules_multi)
     .def("add_hydrogen_atoms",&molecules_container_t::add_hydrogen_atoms)
+    .def("add_lsq_superpose_match",&molecules_container_t::add_lsq_superpose_match)
+    .def("add_lsq_superpose_atom_match",&molecules_container_t::add_lsq_superpose_atom_match)
     .def("add_target_position_restraint",&molecules_container_t::add_target_position_restraint)
     .def("add_target_position_restraint_and_refine",&molecules_container_t::add_target_position_restraint_and_refine)
+    .def("add_terminal_residue_directly",&molecules_container_t::add_terminal_residue_directly)
     .def("add_terminal_residue_directly_using_cid",&molecules_container_t::add_terminal_residue_directly_using_cid)
     .def("add_to_non_drawn_bonds",&molecules_container_t::add_to_non_drawn_bonds)
     .def("add_waters",&molecules_container_t::add_waters)
     .def("all_molecule_contact_dots",&molecules_container_t::all_molecule_contact_dots)
     .def("apply_transformation_to_atom_selection",&molecules_container_t::apply_transformation_to_atom_selection)
-    .def("average_map",&molecules_container_t::average_map)
+    .def("assign_sequence",&molecules_container_t::assign_sequence)
     .def("associate_data_mtz_file_with_map",&molecules_container_t::associate_data_mtz_file_with_map)
+    .def("associate_sequence",&molecules_container_t::associate_sequence)
     .def("auto_fit_rotamer",&molecules_container_t::auto_fit_rotamer)
     .def("auto_read_mtz",&molecules_container_t::auto_read_mtz)
+    .def("average_map",&molecules_container_t::average_map)
     .def("calculate_new_rail_points",&molecules_container_t::calculate_new_rail_points)
     .def("change_to_first_rotamer",&molecules_container_t::change_to_first_rotamer)
     .def("change_to_next_rotamer",&molecules_container_t::change_to_next_rotamer)
     .def("change_to_previous_rotamer",&molecules_container_t::change_to_previous_rotamer)
     .def("cis_trans_convert",&molecules_container_t::cis_trans_convert)
     .def("clear_extra_restraints",&molecules_container_t::clear_extra_restraints)
+    .def("clear_lsq_matches",&molecules_container_t::clear_lsq_matches)
     .def("clear_non_drawn_bonds",&molecules_container_t::clear_non_drawn_bonds)
     .def("clear_refinement",&molecules_container_t::clear_refinement)
     .def("clear_target_position_restraints",&molecules_container_t::clear_target_position_restraints)
+    .def("change_alt_locs",&molecules_container_t::change_alt_locs)
     .def("close_molecule",&molecules_container_t::close_molecule)
     .def("connect_updating_maps",&molecules_container_t::connect_updating_maps)
     .def("contact_dots_for_ligand",&molecules_container_t::contact_dots_for_ligand)
     .def("copy_fragment_for_refinement_using_cid",&molecules_container_t::copy_fragment_for_refinement_using_cid)
     .def("copy_fragment_using_cid",&molecules_container_t::copy_fragment_using_cid)
     .def("copy_fragment_using_residue_range",&molecules_container_t::copy_fragment_using_residue_range)
+    .def("copy_molecule",&molecules_container_t::copy_molecule)
+    .def("delete_all_carbohydrate",&molecules_container_t::delete_all_carbohydrate)
     .def("delete_atom",&molecules_container_t::delete_atom)
     .def("delete_atom_using_cid",&molecules_container_t::delete_atom_using_cid)
     .def("delete_chain_using_cid",&molecules_container_t::delete_chain_using_cid)
@@ -233,10 +246,17 @@ NB_MODULE(chapi, m) {
     .def("delete_using_cid",&molecules_container_t::delete_using_cid)
     .def("density_correlation_analysis",&molecules_container_t::density_correlation_analysis)
     .def("density_fit_analysis",&molecules_container_t::density_fit_analysis)
+    .def("dictionary_atom_name_map",&molecules_container_t::dictionary_atom_name_map)
     .def("difference_map_peaks",&molecules_container_t::difference_map_peaks)
     .def("eigen_flip_ligand", nb::overload_cast<int, const std::string&>                        (&molecules_container_t::eigen_flip_ligand_using_cid))
+    .def("export_chemical_features_as_gltf",&molecules_container_t::export_chemical_features_as_gltf)
+    .def("export_molecular_representation_as_gltf",&molecules_container_t::export_molecular_representation_as_gltf)
+    .def("export_model_molecule_as_gltf",&molecules_container_t::export_model_molecule_as_gltf)
+    .def("export_map_molecule_as_gltf",&molecules_container_t::export_map_molecule_as_gltf)
+    .def("find_water_baddies",&molecules_container_t::find_water_baddies)
     .def("file_name_to_string",&molecules_container_t::file_name_to_string)
     .def("fill_partial_residue",&molecules_container_t::fill_partial_residue)
+    .def("fill_partial_residues",&molecules_container_t::fill_partial_residues)
     .def("fill_rotamer_probability_tables",&molecules_container_t::fill_rotamer_probability_tables)
     .def("fit_ligand",&molecules_container_t::fit_ligand)
     .def("fit_ligand_right_here",&molecules_container_t::fit_ligand_right_here)
@@ -246,11 +266,15 @@ NB_MODULE(chapi, m) {
     .def("flipPeptide",       nb::overload_cast<int, const coot::atom_spec_t&,const std::string&>(&molecules_container_t::flip_peptide))
     .def("flipPeptide_cid",   nb::overload_cast<int, const std::string&,      const std::string&>(&molecules_container_t::flip_peptide_using_cid))
     .def("flip_hand",&molecules_container_t::flip_hand)
+    .def("flood",&molecules_container_t::flood)
     .def("fourier_shell_correlation",&molecules_container_t::fourier_shell_correlation)
     .def("generate_self_restraints",&molecules_container_t::generate_self_restraints)
     .def("geometry_init_standard",&molecules_container_t::geometry_init_standard)
     .def("get_active_atom",&molecules_container_t::get_active_atom)
+    .def("get_acedrg_atom_types",&molecules_container_t::get_acedrg_atom_types)
+    .def("get_acedrg_atom_types_for_ligand",&molecules_container_t::get_acedrg_atom_types_for_ligand)
     .def("get_atom",&molecules_container_t::get_atom, nb::rv_policy::reference)
+    .def("get_atom_using_cid",&molecules_container_t::get_atom_using_cid, nb::rv_policy::reference)
     .def("get_bonds_mesh",&molecules_container_t::get_bonds_mesh)
     .def("get_bonds_mesh_for_selection_instanced",&molecules_container_t::get_bonds_mesh_for_selection_instanced)
     .def("get_bonds_mesh_instanced",&molecules_container_t::get_bonds_mesh_instanced)
@@ -260,45 +284,67 @@ NB_MODULE(chapi, m) {
     .def("get_colour_rules",&molecules_container_t::get_colour_rules)
     .def("get_colour_table_for_blender", &molecules_container_t::get_colour_table_for_blender)
     .def("get_density_at_position", &molecules_container_t::get_density_at_position)
+    .def("get_dictionary_conformers", &molecules_container_t::get_dictionary_conformers)
+    .def("get_distances_between_atoms_of_residues", &molecules_container_t::get_distances_between_atoms_of_residues)
     .def("get_gaussian_surface",&molecules_container_t::get_gaussian_surface)
     .def("get_goodsell_style_mesh_instanced",&molecules_container_t::get_goodsell_style_mesh_instanced)
     .def("get_gphl_chem_comp_info",&molecules_container_t::get_gphl_chem_comp_info)
     .def("get_group_for_monomer",&molecules_container_t::get_group_for_monomer)
     .def("get_groups_for_monomers",&molecules_container_t::get_groups_for_monomers)
     .def("get_hb_type",&molecules_container_t::get_hb_type)
+    .def("get_header_info",&molecules_container_t::get_header_info)
     // .def("get_h_bonds",&molecules_container_t::get_h_bonds)
     // .def("get_interesting_places",&molecules_container_t::get_interesting_places)
+    .def("get_HOLE",&molecules_container_t::get_HOLE)
+    .def("get_imol_enc_any",&molecules_container_t::get_imol_enc_any)
+    .def("get_ligand_validation_vs_dictionary",&molecules_container_t::get_ligand_validation_vs_dictionary)
+    .def("get_ligand_distortion",&molecules_container_t::get_ligand_distortion)
+    .def("get_lsq_matrix",&molecules_container_t::get_lsq_matrix)
     .def("get_map_contours_mesh",&molecules_container_t::get_map_contours_mesh)
     .def("get_map_molecule_centre",&molecules_container_t::get_map_molecule_centre)
     .def("get_map_rmsd_approx",&molecules_container_t::get_map_rmsd_approx)
     .def("get_map_weight",&molecules_container_t::get_map_weight)
+    .def("get_median_temperature_factor",&molecules_container_t::get_median_temperature_factor)
+    .def("get_missing_residue_ranges",&molecules_container_t::get_missing_residue_ranges)
     .def("get_molecular_representation_mesh",&molecules_container_t::get_molecular_representation_mesh)
     .def("get_molecule_centre",&molecules_container_t::get_molecule_centre)
     .def("get_molecule_name",&molecules_container_t::get_molecule_name)
     .def("get_monomer",&molecules_container_t::get_monomer)
     .def("get_monomer_and_position_at",&molecules_container_t::get_monomer_and_position_at)
     .def("get_monomer_from_dictionary",&molecules_container_t::get_monomer_from_dictionary)
+    .def("get_number_of_atoms",&molecules_container_t::get_number_of_atoms)
+    .def("get_number_of_hydrogen_atoms",&molecules_container_t::get_number_of_hydrogen_atoms)
     .def("get_number_of_molecules",&molecules_container_t::get_number_of_molecules)
     .def("get_number_of_map_sections",&molecules_container_t::get_number_of_map_sections)
+    .def("get_octahemisphere",&molecules_container_t::get_octahemisphere)
+    .def("get_q_score",&molecules_container_t::get_q_score)
+    .def("get_q_score_for_cid",&molecules_container_t::get_q_score_for_cid)
     .def("get_r_factor_stats",&molecules_container_t::get_r_factor_stats)
     .def("get_rama_plot_restraints_weight",&molecules_container_t::get_rama_plot_restraints_weight)
     .def("get_ramachandran_validation_markup_mesh",&molecules_container_t::get_ramachandran_validation_markup_mesh)
     //Using allow_raw_pointers(). Perhaps suggests we need to do something different from exposing mmdb pointers to JS.
     .def("get_residue",&molecules_container_t::get_residue, nb::rv_policy::reference)
+    .def("get_residue_average_position",&molecules_container_t::get_residue_average_position)
+    .def("get_residue_CA_position",&molecules_container_t::get_residue_CA_position)
     .def("get_residue_name",&molecules_container_t::get_residue_name)
     .def("get_residue_names_with_no_dictionary",&molecules_container_t::get_residue_names_with_no_dictionary)
+    .def("get_residue_sidechain_average_position",&molecules_container_t::get_residue_sidechain_average_position)
     .def("get_residue_using_cid",&molecules_container_t::get_residue_using_cid)
     .def("get_residues_near_residue",&molecules_container_t::get_residues_near_residue)
     .def("get_rotamer_dodecs",&molecules_container_t::get_rotamer_dodecs)
     .def("get_rotamer_dodecs_instanced",&molecules_container_t::get_rotamer_dodecs_instanced)
     .def("get_single_letter_codes_for_chain",&molecules_container_t::get_single_letter_codes_for_chain)
+    .def("get_sum_density_for_atoms_in_residue",&molecules_container_t::get_sum_density_for_atoms_in_residue)
+    .def("get_svg_for_2d_ligand_environment_view",&molecules_container_t::get_svg_for_2d_ligand_environment_view)
     .def("get_svg_for_residue_type",&molecules_container_t::get_svg_for_residue_type)
     .def("get_symmetry",&molecules_container_t::get_symmetry)
+    .def("get_torsion",&molecules_container_t::get_torsion)
     .def("get_torsion_restraints_weight",&molecules_container_t::get_torsion_restraints_weight)
     .def("get_triangles_for_blender", &molecules_container_t::get_triangles_for_blender)
     .def("get_use_gemmi",&molecules_container_t::get_use_gemmi)
     .def("get_use_rama_plot_restraints",&molecules_container_t::get_use_rama_plot_restraints)
     .def("get_use_torsion_restraints",&molecules_container_t::get_use_torsion_restraints)
+    .def("get_validation_vs_dictionary_for_selection", &molecules_container_t::get_validation_vs_dictionary_for_selection)
     .def("get_vertices_for_blender", &molecules_container_t::get_vertices_for_blender)
     .def("go_to_blob",&molecules_container_t::go_to_blob)
     .def("import_cif_dictionary",&molecules_container_t::import_cif_dictionary)
@@ -306,15 +352,20 @@ NB_MODULE(chapi, m) {
     .def("is_a_difference_map",&molecules_container_t::is_a_difference_map)
     .def("is_valid_map_molecule",&molecules_container_t::is_valid_map_molecule)
     .def("is_valid_model_molecule",&molecules_container_t::is_valid_model_molecule)
-    .def("jed_flip",          nb::overload_cast<int, const std::string&, bool>           (&molecules_container_t::jed_flip))
+    .def("jed_flip",      nb::overload_cast<int, const std::string&, bool> (&molecules_container_t::jed_flip))
+    .def("lsq_superpose", &molecules_container_t::lsq_superpose)
+    .def("make_mask", &molecules_container_t::make_mask)
     .def("make_mesh_for_bonds_for_blender", &molecules_container_t::make_mesh_for_bonds_for_blender)
     .def("make_mesh_for_gaussian_surface_for_blender", &molecules_container_t::make_mesh_for_gaussian_surface_for_blender)
     .def("make_mesh_for_goodsell_style_for_blender", &molecules_container_t::make_mesh_for_goodsell_style_for_blender)
     .def("make_mesh_for_map_contours_for_blender", &molecules_container_t::make_mesh_for_map_contours_for_blender)
     .def("make_mesh_for_molecular_representation_for_blender", &molecules_container_t::make_mesh_for_molecular_representation_for_blender)
     .def("mask_map_by_atom_selection",&molecules_container_t::mask_map_by_atom_selection)
+    .def("make_power_scaled_map", &molecules_container_t::make_power_scaled_map)
     .def("merge_molecules", nb::overload_cast<int,const std::string &>(&molecules_container_t::merge_molecules))
     .def("minimize_energy",&molecules_container_t::minimize_energy)
+    .def("minimize",&molecules_container_t::minimize)
+    .def("mmcif_tests",&molecules_container_t::mmcif_tests)
     .def("mmrrcc",&molecules_container_t::mmrrcc)
     .def("move_molecule_to_new_centre",&molecules_container_t::move_molecule_to_new_centre)
     .def("multiply_residue_temperature_factors",&molecules_container_t::multiply_residue_temperature_factors)
@@ -322,26 +373,41 @@ NB_MODULE(chapi, m) {
     .def("new_positions_for_atoms_in_residues",&molecules_container_t::new_positions_for_atoms_in_residues)
     .def("new_positions_for_residue_atoms",&molecules_container_t::new_positions_for_residue_atoms)
     .def("non_standard_residue_types_in_model",&molecules_container_t::non_standard_residue_types_in_model)
+    .def("package_version",&molecules_container_t::package_version)
+    .def("partition_map_by_chain",&molecules_container_t::partition_map_by_chain)
     .def("pepflips_using_difference_map",&molecules_container_t::pepflips_using_difference_map)
     .def("peptide_omega_analysis",&molecules_container_t::peptide_omega_analysis)
     .def("print_secondary_structure_info",&molecules_container_t::print_secondary_structure_info)
     .def("rail_points_total",&molecules_container_t::rail_points_total)
     .def("ramachandran_analysis",&molecules_container_t::ramachandran_analysis)
     .def("ramachandran_validation",&molecules_container_t::ramachandran_validation)
+    .def("read_coordinates",&molecules_container_t::read_coordinates)
     .def("read_ccp4_map",&molecules_container_t::read_ccp4_map)
+    .def("read_extra_restraints",&molecules_container_t::read_extra_restraints)
     .def("read_mtz",&molecules_container_t::read_mtz)
     .def("read_pdb",&molecules_container_t::read_pdb)
+    .def("read_small_molecule_cif",&molecules_container_t::read_small_molecule_cif)
     .def("redo",&molecules_container_t::redo)
     .def("refine",&molecules_container_t::refine)
     .def("refine_residue_range",&molecules_container_t::refine_residue_range)
+    .def("refine_residues",&molecules_container_t::refine_residues)
     .def("refine_residues_using_atom_cid",&molecules_container_t::refine_residues_using_atom_cid)
     .def("regen_map",&molecules_container_t::regen_map)
     .def("replace_fragment",&molecules_container_t::replace_fragment)
     .def("replace_map_by_mtz_from_file",&molecules_container_t::replace_map_by_mtz_from_file)
     .def("replace_molecule_by_model_from_file",&molecules_container_t::replace_molecule_by_model_from_file)
+    .def("replace_residue",&molecules_container_t::replace_residue)
     .def("residues_with_missing_atoms",&molecules_container_t::residues_with_missing_atoms)
     .def("rigid_body_fit",&molecules_container_t::rigid_body_fit)
     .def("rotamer_analysis",&molecules_container_t::rotamer_analysis)
+    .def("rotate_around_bond",&molecules_container_t::rotate_around_bond)
+    .def("scale_map",&molecules_container_t::scale_map)
+    .def("servalcat_refine_xray",&molecules_container_t::servalcat_refine_xray)
+    .def("servalcat_refine_xray_with_keywords",&molecules_container_t::servalcat_refine_xray_with_keywords)
+    .def("set_add_waters_sigma_cutoff",&molecules_container_t::set_add_waters_sigma_cutoff)
+    .def("set_add_waters_variance_limit",&molecules_container_t::set_add_waters_variance_limit)
+    .def("set_add_waters_water_to_protein_distance_lim_min",&molecules_container_t::set_add_waters_water_to_protein_distance_lim_min)
+    .def("set_add_waters_water_to_protein_distance_lim_max",&molecules_container_t::set_add_waters_water_to_protein_distance_lim_max)
     .def("set_colour_wheel_rotation_base",&molecules_container_t::set_colour_wheel_rotation_base)
     .def("set_draw_missing_residue_loops",&molecules_container_t::set_draw_missing_residue_loops)
     .def("set_draw_missing_residue_loops",&molecules_container_t::set_draw_missing_residue_loops)
@@ -349,11 +415,14 @@ NB_MODULE(chapi, m) {
     .def("set_make_backups",&molecules_container_t::set_make_backups)
     .def("set_map_sampling_rate",&molecules_container_t::set_map_sampling_rate)
     .def("set_map_weight",&molecules_container_t::set_map_weight)
+    .def("set_max_number_of_threads",&molecules_container_t::set_max_number_of_threads)
     .def("set_molecule_name",&molecules_container_t::set_molecule_name)
+    .def("set_occupancy",&molecules_container_t::set_occupancy)
     .def("set_rama_plot_restraints_weight",&molecules_container_t::set_rama_plot_restraints_weight)
     .def("set_refinement_is_verbose",&molecules_container_t::set_refinement_is_verbose)
-    .def("set_refinement_is_verbose",&molecules_container_t::set_refinement_is_verbose)
+    .def("set_refinement_geman_mcclure_alpha",&molecules_container_t::set_refinement_geman_mcclure_alpha)
     .def("set_show_timings",&molecules_container_t::set_show_timings)
+    .def("set_temperature_factors_using_cid",&molecules_container_t::set_temperature_factors_using_cid)
     .def("set_torsion_restraints_weight",&molecules_container_t::set_torsion_restraints_weight)
     .def("set_use_gemmi",&molecules_container_t::set_use_gemmi)
     .def("set_use_rama_plot_restraints",&molecules_container_t::set_use_rama_plot_restraints)
@@ -366,13 +435,28 @@ NB_MODULE(chapi, m) {
     .def("sharpen_blur_map_with_resample",&molecules_container_t::sharpen_blur_map_with_resample)
     .def("side_chain_180",    nb::overload_cast<int, const std::string&>                         (&molecules_container_t::side_chain_180))
     .def("split_multi_model_molecule",&molecules_container_t::split_multi_model_molecule)
+    .def("split_residue_using_map",&molecules_container_t::split_residue_using_map)
+    .def("test_function",&molecules_container_t::test_function)
     .def("test_origin_cube",&molecules_container_t::test_origin_cube)
+    .def("transform_map_using_lsq_matrix",&molecules_container_t::transform_map_using_lsq_matrix)
     .def("undo",&molecules_container_t::undo)
     .def("unmodelled_blobs",&molecules_container_t::unmodelled_blobs)
     .def("write_coordinates",&molecules_container_t::write_coordinates)
     .def("write_map",&molecules_container_t::write_map)
     .def("write_png",&molecules_container_t::write_png)
     ;
+    nb::class_<coot::chain_mutation_info_container_t>(m,"chain_mutation_info_container_t")
+      .def_ro("chain_id",         &coot::chain_mutation_info_container_t::chain_id)
+      .def_ro("alignedS",         &coot::chain_mutation_info_container_t::alignedS)
+      .def_ro("alignedT",         &coot::chain_mutation_info_container_t::alignedT)
+      .def_ro("alignedS_label",   &coot::chain_mutation_info_container_t::alignedS_label)
+      .def_ro("alignedT_label",   &coot::chain_mutation_info_container_t::alignedT_label)
+      .def_ro("alignment_string", &coot::chain_mutation_info_container_t::alignment_string)
+      .def_ro("alignment_score",  &coot::chain_mutation_info_container_t::alignment_score)
+      .def_ro("insertions",       &coot::chain_mutation_info_container_t::insertions)
+      .def_ro("deletions",        &coot::chain_mutation_info_container_t::deletions)
+      .def_ro("mutations",        &coot::chain_mutation_info_container_t::mutations)
+      ;
     nb::class_<molecules_container_js, molecules_container_t>(m,"molecules_container_py")
     .def(nb::init<bool>())
     .def("writePDBASCII",&molecules_container_js::writePDBASCII)
@@ -389,6 +473,16 @@ NB_MODULE(chapi, m) {
     .def_prop_ro("spec",[](merge_molecule_results_info_t &t) { return t.spec ; })
     .def_ro("is_chain", &merge_molecule_results_info_t::is_chain)
     ;
+    nb::enum_<coot::graph_data_type>(m, "graph_data_type")
+       .value("Unset",          coot::graph_data_type::UNSET)
+       .value("Density",        coot::graph_data_type::DENSITY)
+       .value("Distortion",     coot::graph_data_type::DISTORTION)
+       .value("Energy",         coot::graph_data_type::ENERGY)
+       .value("Probability",    coot::graph_data_type::PROBABILITY)
+       .value("Correlation",    coot::graph_data_type::CORRELATION)
+       .value("LogProbability", coot::graph_data_type::LOG_PROBABILITY)
+       .value("TorsionAngle",   coot::graph_data_type::TORSION_ANGLE)
+       ;
     nb::class_<coot::residue_validation_information_t>(m,"residue_validation_information_t")
     .def_ro("function_value", &coot::residue_validation_information_t::function_value)
     .def_ro("label", &coot::residue_validation_information_t::label)
@@ -405,6 +499,34 @@ NB_MODULE(chapi, m) {
     .def_ro("cviv", &coot::validation_information_t::cviv)
     .def("get_index_for_chain",&coot::validation_information_t::get_index_for_chain)
     ;
+    nb::enum_<coot::restraint_type_t>(m, "restraint_type")
+       .value("Bond", coot::restraint_type_t::BOND_RESTRAINT)
+       .value("Angle", coot::restraint_type_t::ANGLE_RESTRAINT)
+       .value("Torsion", coot::restraint_type_t::TORSION_RESTRAINT)
+       .value("Plane", coot::restraint_type_t::PLANE_RESTRAINT)
+       .value("Non-Bonded-Contact", coot::restraint_type_t::NON_BONDED_CONTACT_RESTRAINT)
+       .value("Chiral-Volume", coot::restraint_type_t::CHIRAL_VOLUME_RESTRAINT)
+       .value("Trans-Peptide", coot::restraint_type_t::TRANS_PEPTIDE_RESTRAINT)
+       .value("Geman-McClure", coot::restraint_type_t::GEMAN_MCCLURE_DISTANCE_RESTRAINT)// add start pos and target pos at some stage
+       ;
+    nb::class_<coot::simple_restraint>(m, "simple_restraint")
+       .def_ro("restraint_type", &coot::simple_restraint::restraint_type)
+       .def_ro("target_value",   &coot::simple_restraint::target_value)
+    ;
+    nb::class_<coot::geometry_distortion_info_t>(m, "geometry_distortion_info_t")
+       .def_ro("distortion_score",  &coot::geometry_distortion_info_t::distortion_score)
+       .def_ro("atom_indices",      &coot::geometry_distortion_info_t::atom_indices)
+       .def_ro("atom_specs",        &coot::geometry_distortion_info_t::atom_specs)
+       .def_ro("residue_spec",      &coot::geometry_distortion_info_t::residue_spec)
+       .def_ro("restraint",         &coot::geometry_distortion_info_t::restraint)
+       ;
+    nb::class_<coot::geometry_distortion_info_container_t>(m, "geometry_distortion_info_container_t")
+       .def_ro("chain_id",            &coot::geometry_distortion_info_container_t::chain_id)
+       .def("distortion_sum",         &coot::geometry_distortion_info_container_t::distortion_sum)
+       .def("size",                   &coot::geometry_distortion_info_container_t::size)
+       .def("get_geometry_distortion_info", &coot::geometry_distortion_info_container_t::get_geometry_distortion_info)
+       .def_ro("geometry_distortion", &coot::geometry_distortion_info_container_t::geometry_distortion)
+    ;
     nb::class_<molecules_container_t::fit_ligand_info_t>(m, "fit_ligand_info_t")
     .def_ro("imol", &molecules_container_t::fit_ligand_info_t::imol)
     .def_ro("cluster_idx", &molecules_container_t::fit_ligand_info_t::cluster_idx)
@@ -432,6 +554,17 @@ NB_MODULE(chapi, m) {
     .def_rw("string_user_data",&coot::atom_spec_t::string_user_data)
     .def_rw("model_number",&coot::atom_spec_t::model_number)
     ;
+    nb::class_<coot::atom_distance_t>(m,"atom_distance_t")
+      .def_ro("atom_1", &coot::atom_distance_t::atom_1)
+      .def_ro("atom_2", &coot::atom_distance_t::atom_2)
+      .def_ro("distance", &coot::atom_distance_t::distance)
+      ;
+    nb::class_<coot::residue_range_t>(m,"residue_range_t")
+      .def(nb::init<>())
+      .def_rw("chain_id",     &coot::residue_range_t::chain_id)
+      .def_rw("res_no_start", &coot::residue_range_t::res_no_start)
+      .def_rw("res_no_end",   &coot::residue_range_t::res_no_end)
+      ;
     nb::class_<generic_3d_lines_bonds_box_t>(m,"generic_3d_lines_bonds_box_t")
     .def_ro("line_segments", &generic_3d_lines_bonds_box_t::line_segments)
     ;
@@ -522,6 +655,17 @@ NB_MODULE(chapi, m) {
     .def_ro("geom",   &coot::instanced_mesh_t::geom)
     .def_ro("markup", &coot::instanced_mesh_t::markup)
     ;
+    nb::class_<coot::acedrg_types_for_bond_t>(m,"acedrg_types_for_bond_t")
+       .def_ro("atom_id_1",   &coot::acedrg_types_for_bond_t::atom_id_1)
+       .def_ro("atom_id_2",   &coot::acedrg_types_for_bond_t::atom_id_2)
+       .def_ro("atom_type_1", &coot::acedrg_types_for_bond_t::atom_type_1)
+       .def_ro("atom_type_2", &coot::acedrg_types_for_bond_t::atom_type_2)
+       .def_ro("bond_length", &coot::acedrg_types_for_bond_t::bond_length)
+       .def_ro("bond_is_between_atoms_in_the_same_ring", &coot::acedrg_types_for_bond_t::bond_is_between_atoms_in_the_same_ring)
+    ;
+    nb::class_<coot::acedrg_types_for_residue_t>(m,"acedrg_types_for_residue_t")
+        .def_ro("bond_types", &coot::acedrg_types_for_residue_t::bond_types)
+    ;
     nb::class_<coot::util::phi_psi_t>(m,"phi_psi_t")
     .def("phi",               &coot::util::phi_psi_t::phi)
     .def("psi",               &coot::util::phi_psi_t::psi)
@@ -630,6 +774,29 @@ NB_MODULE(chapi, m) {
     //    .def_ro("normals",   &coot::blender_mesh_t::normals)
     //    .def_ro("triangles", &coot::blender_mesh_t::triangles)
     // ;
+    nb::class_<moorhen::helix_t>(m,"helix_t")
+       .def_ro("serNum", &moorhen::helix_t::serNum)
+       .def_ro("helixID", &moorhen::helix_t::helixID)
+       .def_ro("initResName", &moorhen::helix_t::initResName)
+       .def_ro("initChainID", &moorhen::helix_t::initChainID)
+       .def_ro("initSeqNum", &moorhen::helix_t::initSeqNum)
+       .def_ro("initICode", &moorhen::helix_t::initICode)
+       .def_ro("endResName", &moorhen::helix_t::endResName)
+       .def_ro("endChainID", &moorhen::helix_t::endChainID)
+       .def_ro("endSeqNum", &moorhen::helix_t::endSeqNum)
+       .def_ro("endICode", &moorhen::helix_t::endICode)
+       .def_ro("helixClass", &moorhen::helix_t::helixClass)
+       .def_ro("comment", &moorhen::helix_t::comment)
+       .def_ro("length", &moorhen::helix_t::length)
+       ;
+
+    nb::class_<moorhen::header_info_t>(m,"header_info_t")
+       .def_ro("title", &moorhen::header_info_t::title)
+       .def_ro("author_lines", &moorhen::header_info_t::author_lines)
+       .def_ro("journal_lines", &moorhen::header_info_t::journal_lines)
+       .def_ro("compound_lines", &moorhen::header_info_t::compound_lines)
+       .def_ro("helix_info", &moorhen::header_info_t::helix_info)
+       ;
 
     nb::class_<coot::util::density_correlation_stats_info_t>(m,"density_correlation_stats_info_t")
     .def_ro("n",          &coot::util::density_correlation_stats_info_t::n)
@@ -650,6 +817,11 @@ NB_MODULE(chapi, m) {
        .def_ro("aligned_pairs",      &superpose_results_t::aligned_pairs)
     ;
 
+    nb::class_<lsq_results_t>(m, "lsq_results_t")
+      .def_ro("rotation_matrix", &lsq_results_t::rotation_matrix)
+      .def_ro("translation",     &lsq_results_t::translation)
+   ;
+
     nb::class_<moorhen::h_bond>(m,"h_bond")
         .def_ro("hb_hydrogen",&moorhen::h_bond::hb_hydrogen)
         .def_ro("donor",&moorhen::h_bond::donor)
@@ -688,14 +860,15 @@ NB_MODULE(chapi, m) {
     .def("residue_name", &coot::phi_psi_prob_t::residue_name)
     .def("is_allowed", &coot::phi_psi_prob_t::is_allowed)
     ;
-    nb::class_<coot::molecule_t::moved_atom_t>(m,"moved_atom_t")
+    nb::class_<coot::api::moved_atom_t>(m,"moved_atom_t")
+    .def(nb::init<const std::string&, const std::string&, float, float, float>())
     .def(nb::init<const std::string&, const std::string&, float, float, float, int>())
-    .def_ro("atom_name", &coot::molecule_t::moved_atom_t::atom_name)
-    .def_ro("alt_conf", &coot::molecule_t::moved_atom_t::alt_conf)
-    .def_ro("x", &coot::molecule_t::moved_atom_t::x)
-    .def_ro("y", &coot::molecule_t::moved_atom_t::y)
-    .def_ro("z", &coot::molecule_t::moved_atom_t::z)
-    .def_ro("index", &coot::molecule_t::moved_atom_t::index)
+    .def_ro("atom_name", &coot::api::moved_atom_t::atom_name)
+    .def_ro("alt_conf", &coot::api::moved_atom_t::alt_conf)
+    .def_ro("x", &coot::api::moved_atom_t::x)
+    .def_ro("y", &coot::api::moved_atom_t::y)
+    .def_ro("z", &coot::api::moved_atom_t::z)
+    .def_ro("index", &coot::api::moved_atom_t::index)
     ;
     nb::class_<coot::molecule_t::interesting_place_t>(m,"interesting_place_t")
     .def(nb::init<const std::string &, const coot::residue_spec_t &, const clipper::Coord_orth &, const std::string &>())
@@ -709,12 +882,12 @@ NB_MODULE(chapi, m) {
     .def_ro("y", &coot::molecule_t::interesting_place_t::y)
     .def_ro("z", &coot::molecule_t::interesting_place_t::z)
     ;
-    nb::class_<coot::molecule_t::moved_residue_t>(m,"moved_residue_t")
+    nb::class_<coot::api::moved_residue_t>(m,"moved_residue_t")
     .def(nb::init<const std::string&, int, const std::string&>())
-    .def_ro("chain_id", &coot::molecule_t::moved_residue_t::chain_id)
-    .def_ro("res_no", &coot::molecule_t::moved_residue_t::res_no)
-    .def_ro("ins_code", &coot::molecule_t::moved_residue_t::ins_code)
-    .def_ro("moved_atoms", &coot::molecule_t::moved_residue_t::moved_atoms)
-    .def("add_atom",&coot::molecule_t::moved_residue_t::add_atom)
+    .def_ro("chain_id", &coot::api::moved_residue_t::chain_id)
+    .def_ro("res_no", &coot::api::moved_residue_t::res_no)
+    .def_ro("ins_code", &coot::api::moved_residue_t::ins_code)
+    .def_ro("moved_atoms", &coot::api::moved_residue_t::moved_atoms)
+    .def("add_atom",&coot::api::moved_residue_t::add_atom)
     ;
 }
