@@ -25,7 +25,9 @@
  */
 #define _USE_MATH_DEFINES
 #include <iostream>
+#include <iomanip>
 #include <vector>
+#include <chrono>
 
 // #include <glm/ext.hpp>
 #define GLM_ENABLE_EXPERIMENTAL
@@ -34,6 +36,7 @@
 
 #include "prideout-octasphere.hh"
 #include "oct.hh"
+#include "ortep.hh"
 
 std::vector<g_triangle>
 make_octasphere_triangles(unsigned int i_row,
@@ -73,39 +76,6 @@ make_octasphere_triangles(unsigned int i_row,
 }
 
 
-std::pair<std::vector<glm::vec3>, std::vector<g_triangle> >
-tessellate_hemisphere_patch(unsigned int num_subdivisions) {
-
-   std::vector<glm::vec3> verts;
-   std::vector<g_triangle> triangles;
-
-   std::pair<std::vector<glm::vec3>, std::vector<g_triangle> > t =
-      tessellate_octasphere_patch(num_subdivisions);
-
-   // rotate t 4 times to make a hemisphere
-
-   verts = t.first;
-   triangles = t.second;
-
-   for (unsigned int irot=1; irot<4; irot++) {
-      float angle = 0.5f * M_PI * static_cast<float>(irot);
-      unsigned int idx_base = verts.size();
-      unsigned int idx_triangle_base = triangles.size();
-      for (unsigned int i=0; i<t.first.size(); i++) {
-         glm::vec3 v = glm::rotate(t.first[i], angle, glm::vec3(0,0,1));
-         verts.push_back(v);
-      }
-      // reindex the triangles
-      triangles.insert(triangles.end(), t.second.begin(), t.second.end());
-      for (unsigned int i=idx_triangle_base; i<triangles.size(); i++)
-         triangles[i].rebase(idx_base);
-   }
-
-   return std::pair<std::vector<glm::vec3>, std::vector<g_triangle> > (verts,triangles);
-}
-
-#include<algorithm>
-
 // adjust triangles_p - the vertices are not changed - so if this is successful,
 // then vertices that are not used will be sent to the graphics card.
 // Not many though, in the scheme of things. Also, fixing the vertex indexing might be painful.
@@ -123,6 +93,8 @@ remove_redundant_vertices(std::vector<glm::vec3> *vertices_p,
    //              << tri.point_id[2] << " " << std::endl;
    // }
 
+   bool debug = false;
+
    std::map<unsigned int, std::set<unsigned int> >::const_iterator it;
    unsigned int n_done = 0;
    for (it=redundant_map.begin(); it!=redundant_map.end(); ++it) {
@@ -166,7 +138,7 @@ remove_redundant_vertices(std::vector<glm::vec3> *vertices_p,
       }
    }
 
-   if (false) {
+   if (debug) {
       std::cout << "DEBUG:: remove_redundant_vertices(): Here are the accumulated_unused_vertices: " << std::endl;
       std::set<unsigned int>::const_iterator it_s;
       for (it_s=accumulated_unused_vertices.begin(); it_s!=accumulated_unused_vertices.end(); ++it_s)
@@ -188,7 +160,7 @@ remove_redundant_vertices(std::vector<glm::vec3> *vertices_p,
       }
    }
 
-   if (false) { // debugging
+   if (debug) { // debugging
       int new_size = vertices_p->size() - accumulated_unused_vertices.size();
       std::cout << "DEBUG:: remove_redundant_vertices(): new vertices size " << new_size << std::endl;
       for (map_it=old_index_to_new_index_map.begin(); map_it!=old_index_to_new_index_map.end(); ++map_it) {
@@ -210,10 +182,61 @@ remove_redundant_vertices(std::vector<glm::vec3> *vertices_p,
          tri.point_id[ii] = old_index_to_new_index_map[tri.point_id[ii]];
    }
 
-   // std::cout << "DEBUG:: remove_redundant_vertices(): n_done " << n_done << std::endl;
+   if (debug)
+      std::cout << "DEBUG:: remove_redundant_vertices(): n_done " << n_done << std::endl;
+}
+
+
+std::pair<std::vector<glm::vec3>, std::vector<g_triangle> >
+tessellate_hemisphere_patch(unsigned int num_subdivisions) {
+
+   bool debug = false;
+   bool remove_redundant_vertices_flag = true;
+   std::vector<glm::vec3> verts;
+   std::vector<g_triangle> triangles;
+
+   std::pair<std::vector<glm::vec3>, std::vector<g_triangle> > t =
+      tessellate_octasphere_patch(num_subdivisions);
+
+   // rotate t 4 times to make a hemisphere
+
+   verts = t.first;
+   triangles = t.second;
+
+   for (unsigned int irot=1; irot<4; irot++) {
+      float angle = 0.5f * M_PI * static_cast<float>(irot);
+      unsigned int idx_base = verts.size();
+      unsigned int idx_triangle_base = triangles.size();
+      for (unsigned int i=0; i<t.first.size(); i++) {
+         glm::vec3 v = glm::rotate(t.first[i], angle, glm::vec3(0,0,1));
+         verts.push_back(v);
+      }
+      // reindex the triangles
+      triangles.insert(triangles.end(), t.second.begin(), t.second.end());
+      for (unsigned int i=idx_triangle_base; i<triangles.size(); i++)
+         triangles[i].rebase(idx_base);
+   }
+
+   if (remove_redundant_vertices_flag) {
+      std::map<unsigned int, std::set<unsigned int> > redundant_map = find_same_vertices(verts);
+
+      if (debug) { // debugging
+         std::cout << "debug:: in tessellate_octasphere_patch() with redundant_map size " << redundant_map.size() << std::endl;
+         std::map<unsigned int, std::set<unsigned int> >::const_iterator it;
+         for (it=redundant_map.begin(); it!=redundant_map.end(); ++it) {
+            std::cout << "   " << it->first << ": ";
+            for (const auto its : it->second)
+               std::cout << its << " ";
+            std::cout << std::endl;
+         }
+      }
+
+      // OK! now adjust vertices and triangles
+      remove_redundant_vertices(&verts, &triangles, redundant_map);
+   }
+   return std::pair<std::vector<glm::vec3>, std::vector<g_triangle> > (verts,triangles);
 }
 
-#include <chrono>
 
 std::pair<std::vector<glm::vec3>, std::vector<g_triangle> >
 tessellate_octasphere(unsigned int num_subdivisions, bool remove_redundant_vertices_flag) {
@@ -606,8 +629,6 @@ tessellate_sphere_sans_octant() {
    return ortep;
 }
 
-#include <iomanip>
-
 void
 ortep_t::make_lines() {
 
