@@ -21,6 +21,7 @@
 
 #include "model.hpp"
 #include "render.hpp"
+#include "../qed.hpp"
 #include <exception>
 #include <iterator>
 #include <memory>
@@ -29,10 +30,6 @@
 #include <stdexcept>
 #include <algorithm>
 #include <set>
-// #include <rdkit/GraphMol/MolDraw2D/MolDraw2D.h>
-// // #include <rdkit/GraphMol/MolDraw2D/MolDraw2DCairo.h>
-// // #include <rdkit/GraphMol/MolDraw2D/MolDraw2DSVG.h>
-// #include <rdkit/GraphMol/MolDraw2D/MolDraw2DUtils.h>
 #include <rdkit/GraphMol/Depictor/RDDepictor.h>
 #include <rdkit/GraphMol/Substruct/SubstructMatch.h>
 #include <rdkit/Geometry/point.h>
@@ -168,6 +165,10 @@ graphene_rect_t CanvasMolecule::get_on_screen_bounding_rect() const noexcept {
     return ret;
 }
 
+std::optional<CanvasMolecule::QEDInfo> CanvasMolecule::get_qed_info() const noexcept {
+    return this->qed_info;
+}
+
 
 void CanvasMolecule::perform_flip(FlipMode flip_mode) {
     for(auto& atom: this->cached_atom_coordinate_map.value()) {
@@ -189,6 +190,44 @@ void CanvasMolecule::rotate_by_angle(double radians) {
     }
 }
 
+std::tuple<float,float,float> CanvasMolecule::hightlight_to_rgb(CanvasMolecule::HighlightType htype) noexcept {
+    switch (htype) {
+        case CanvasMolecule::HighlightType::Edition: {
+            return std::make_tuple(1.0, 0.5, 1.0);
+        }
+        case CanvasMolecule::HighlightType::Error: {
+            return std::make_tuple(1.0, 0.0, 0.0);
+        }
+        case CanvasMolecule::HighlightType::Selection: {
+            return std::make_tuple(0.0, 0.75, 1.0);
+        }
+        default:
+        case CanvasMolecule::HighlightType::Hover: {
+            return std::make_tuple(0.0, 1.0, 0.5);
+        }
+    }
+}
+
+std::optional<CanvasMolecule::HighlightType> CanvasMolecule::determine_dominant_highlight(CanvasMolecule::highlight_t hcode) noexcept {
+    if (hcode == 0) {
+        return std::nullopt;
+    }
+    auto has_highlight = [hcode](HighlightType h){
+        return hcode & static_cast<highlight_t>(h);
+    };
+
+    if(has_highlight(HighlightType::Hover)) {
+        return HighlightType::Hover;
+    } else if(has_highlight(HighlightType::Edition)) {
+        return HighlightType::Edition;
+    } else if(has_highlight(HighlightType::Selection)) {
+        return HighlightType::Selection;
+    } else if(has_highlight(HighlightType::Error)) {
+        return HighlightType::Error;
+    }
+    return std::nullopt;
+}
+
 std::tuple<float,float,float> CanvasMolecule::atom_color_to_rgb(CanvasMolecule::AtomColor color) noexcept {
     switch (color) {
         case AtomColor::Green:{
@@ -328,13 +367,13 @@ void CanvasMolecule::draw(impl::Renderer& ren, DisplayMode display_mode) const n
     renctx.draw_bonds();
 }
 
-CanvasMolecule::CanvasMolecule(std::shared_ptr<RDKit::RWMol> rdkit_mol) {
+CanvasMolecule::CanvasMolecule(std::shared_ptr<RDKit::RWMol> rdkit_mol, bool allow_invalid_mol) {
     this->rdkit_molecule = std::move(rdkit_mol);
     this->cached_atom_coordinate_map = std::nullopt;
-    this->lower_from_rdkit(true);
+    this->bounding_atom_coords = std::make_pair(RDGeom::Point2D(0,0),RDGeom::Point2D(0,0));
+    this->lower_from_rdkit(!allow_invalid_mol);
     this->x_canvas_translation = 0;
     this->y_canvas_translation = 0;
-    this->bounding_atom_coords = std::make_pair(RDGeom::Point2D(0,0),RDGeom::Point2D(0,0));
 }
 
 void CanvasMolecule::update_source_molecule(std::shared_ptr<RDKit::RWMol> rdkit_mol) {
@@ -561,9 +600,10 @@ void CanvasMolecule::process_alignment_in_rings() {
         int j = 1;
 
         // Go over every bond
-        while(i!=ring.size()) {
-            int atom_one_idx = ring[i];
-            int atom_two_idx = ring[j];
+        int ring_size = ring.size();
+        while (i != ring_size) {
+            unsigned int atom_one_idx = ring[i];
+            unsigned int atom_two_idx = ring[j];
             auto bonds_of_atom_one = this->bond_map.find(atom_one_idx);
             if(bonds_of_atom_one == this->bond_map.end()) {
                 throw std::runtime_error("Critical internal error: Could not find a bond while processing rings.");
@@ -573,19 +613,19 @@ void CanvasMolecule::process_alignment_in_rings() {
                 return (bond->first_atom_idx == atom_one_idx && bond->second_atom_idx == atom_two_idx) 
                     || (bond->first_atom_idx == atom_two_idx && bond->second_atom_idx == atom_one_idx);
             });
-            if(bond == bonds_of_atom_one->second.end()) {
+            if (bond == bonds_of_atom_one->second.end()) {
                 throw std::runtime_error("Critical internal error: Could not find a bond while processing rings.");
             }
             auto bond_ptr = *bond;
-            if(bond_ptr->type == BondType::Double) {
+            if (bond_ptr->type == BondType::Double) {
                 float x_offset_from_center = (bond_ptr->first_atom_x + bond_ptr->second_atom_x) / 2.f - ring_center_x;
                 // negative y on screen is actually "higher" so we need to flip the sign
                 float y_offset_from_center = ring_center_y - (bond_ptr->first_atom_y + bond_ptr->second_atom_y) / 2.f;
                 bool sign_of_x_offset_from_center = x_offset_from_center > 0.f;
                 bool sign_of_y_offset_from_center = y_offset_from_center > 0.f;
-                bool x_requirement = bond_ptr->second_atom_x > bond_ptr->first_atom_x == sign_of_y_offset_from_center;
+                bool x_requirement = (bond_ptr->second_atom_x > bond_ptr->first_atom_x) == sign_of_y_offset_from_center;
                 // negative y on screen is actually "higher" so we need to flip the sign
-                bool y_requirement = bond_ptr->second_atom_y <= bond_ptr->first_atom_y != sign_of_x_offset_from_center;
+                bool y_requirement = (bond_ptr->second_atom_y <= bond_ptr->first_atom_y) != sign_of_x_offset_from_center;
                 bool bond_direction = x_requirement && y_requirement;
                 // g_debug(
                 //     "Bond: %i->%i DeltaX: %f DeltaY: %f CX: %f CY: %f XO: %f SignXO: %i YO: %f SignYO: %i ReqX: %i ReqY: %i DIR: %i",
@@ -605,25 +645,24 @@ void CanvasMolecule::process_alignment_in_rings() {
             }
             // Lastly, process appendices' alignment relative to ring center
             auto& atom = this->atoms.at(atom_one_idx);
-            if(atom.appendix.has_value()) {
+            if (atom.appendix.has_value()) {
                 bool should_reverse = atom.x < ring_center_x;
                 atom.appendix->reversed = should_reverse;
             }
             i++;
             j++;
             // Process the last bond
-            if(j==ring.size()) {
+            if (j == ring_size) {
                 // Loop j to point to the first atom in the ring
                 j = 0;
             }
         }
-
     }
 }
 
 void CanvasMolecule::shorten_double_bonds() {
     typedef std::pair<const Bond*,float> bond_ptr_and_angle;
-    for(auto& bond: this->bonds) {
+    for (auto& bond: this->bonds) {
         if(bond->type != BondType::Double) {
             continue;
         }
@@ -735,7 +774,8 @@ void CanvasMolecule::build_internal_molecule_representation(const RDGeom::INT_PO
     std::set<unsigned int> processed_atoms_indices;
     
     // 1. Process atoms and compute bonds
-    for(const auto& [atom_idx,plane_point]: coordinate_map) {
+    for (const auto& [atom_idx_i, plane_point] : coordinate_map) {
+        unsigned int atom_idx = atom_idx_i; // just changing signedness
         const auto* rdkit_atom = this->rdkit_molecule->getAtomWithIdx(atom_idx);
         auto canvas_atom = CanvasMolecule::Atom();
         canvas_atom.color = atom_color_from_rdkit(rdkit_atom);
@@ -744,7 +784,7 @@ void CanvasMolecule::build_internal_molecule_representation(const RDGeom::INT_PO
             rdkit_atom->getProp("name", atom_name);
             canvas_atom.name = atom_name;
         }
-        canvas_atom.highlighted = false;
+        canvas_atom.highlight = 0;
         canvas_atom.idx = atom_idx;
         canvas_atom.symbol = rdkit_atom->getSymbol();
         canvas_atom.x = plane_point.x;
@@ -777,13 +817,13 @@ void CanvasMolecule::build_internal_molecule_representation(const RDGeom::INT_PO
         // Used to determine if the 'appendix' should be 'reversed'
         std::optional<float> x_coordinate_of_bonded_atom;
 
-        for(const auto& bond: boost::make_iterator_range(this->rdkit_molecule->getAtomBonds(rdkit_atom))) {
+        for (const auto& bond: boost::make_iterator_range(this->rdkit_molecule->getAtomBonds(rdkit_atom))) {
             // Based on `getAtomBonds` documentation.
             // Seems weird but we have to do it that way.
             const auto* bond_ptr = (*this->rdkit_molecule)[bond];
-            auto first_atom_idx = bond_ptr->getBeginAtomIdx();
+            auto  first_atom_idx = bond_ptr->getBeginAtomIdx();
             auto second_atom_idx = bond_ptr->getEndAtomIdx();
-            auto the_other_atom_idx = first_atom_idx == atom_idx ? second_atom_idx : first_atom_idx;
+            auto the_other_atom_idx = (first_atom_idx == atom_idx) ? second_atom_idx : first_atom_idx;
             const auto* the_other_atom =  this->rdkit_molecule->getAtomWithIdx(the_other_atom_idx);
             if(the_other_atom->getSymbol() != "H") {
                 surrounding_non_hydrogen_count++;
@@ -811,7 +851,7 @@ void CanvasMolecule::build_internal_molecule_representation(const RDGeom::INT_PO
             canvas_bond.second_atom_x = coordinate_map.at(second_atom_idx).x;
             canvas_bond.second_atom_y = coordinate_map.at(second_atom_idx).y;
 
-            canvas_bond.highlighted = false;
+            canvas_bond.highlight = 0;
             canvas_bond.type = bond_type_from_rdkit(bond_ptr->getBondType());
             canvas_bond.geometry = bond_geometry_from_rdkit(bond_ptr->getBondDir());
 
@@ -830,7 +870,7 @@ void CanvasMolecule::build_internal_molecule_representation(const RDGeom::INT_PO
         }
 
         auto cached_bonds_for_this_atom = this->bond_map.find(atom_idx);
-        if(cached_bonds_for_this_atom == this->bond_map.end()) {
+        if (cached_bonds_for_this_atom == this->bond_map.end()) {
             this->bond_map.emplace(std::pair(atom_idx,std::move(bonds_to_be_cached)));
         } else {
             // In the for-loop above,
@@ -840,8 +880,8 @@ void CanvasMolecule::build_internal_molecule_representation(const RDGeom::INT_PO
             std::move(bonds_to_be_cached.begin(),bonds_to_be_cached.end(),std::back_inserter(cached_bonds_for_this_atom->second));
         }
 
-        bool terminus = surrounding_non_hydrogen_count < 2;
-        if(canvas_atom.symbol != "H" && (canvas_atom.symbol != "C" || terminus)) {
+        bool terminus = (surrounding_non_hydrogen_count < 2);
+        if (canvas_atom.symbol != "H" && (canvas_atom.symbol != "C" || terminus)) {
             //todo: oxygens I guess?
             if(surrounding_hydrogen_count > 0) {
                 Atom::Appendix ap = canvas_atom.appendix.value_or(Atom::Appendix());
@@ -856,21 +896,23 @@ void CanvasMolecule::build_internal_molecule_representation(const RDGeom::INT_PO
                 canvas_atom.appendix = ap;
             }
         }
-        auto setup_potential_centered_double_bond = [&](const int atom_idx){
+
+        auto setup_potential_centered_double_bond = [&] (const unsigned int &atom_idx) {
+
             auto bonds_of_this_atom = this->bond_map.find(atom_idx);
-            if(bonds_of_this_atom == this->bond_map.end()) {
+            if (bonds_of_this_atom == this->bond_map.end()) {
                 return;
             }
-            if(!terminus || bonds_of_this_atom->second.empty()) {
+            if (!terminus || bonds_of_this_atom->second.empty()) {
                 return;
             }
             // This means that we only have one bond
             Bond* bond = bonds_of_this_atom->second.front().get();
-            if(bond->type != BondType::Double) {
+            if (bond->type != BondType::Double) {
                 return;
             }
-            unsigned int the_other_atom_idx = bond->first_atom_idx == atom_idx ? bond->second_atom_idx : bond->first_atom_idx;
-            if(this->rdkit_molecule->getAtomWithIdx(the_other_atom_idx)->getAtomicNum() == 6) {
+            unsigned int the_other_atom_idx = (bond->first_atom_idx == atom_idx) ? bond->second_atom_idx : bond->first_atom_idx;
+            if (this->rdkit_molecule->getAtomWithIdx(the_other_atom_idx)->getAtomicNum() == 6) {
                 // This should always be a valid iterator at this point
                 auto bonds_of_the_other_atom = this->bond_map.find(the_other_atom_idx);
                 if(bonds_of_the_other_atom->second.size() != 3 && bonds_of_the_other_atom->second.size() != 1) {
@@ -894,52 +936,148 @@ void CanvasMolecule::build_internal_molecule_representation(const RDGeom::INT_PO
     this->shorten_double_bonds();
 }
 
-void CanvasMolecule::lower_from_rdkit(bool sanitize_after) {
+void CanvasMolecule::lower_from_rdkit(bool sanitize_after, bool with_qed) {
 
     // 2. Do the lowering
 
     // 2.0 Kekulize
-    RDKit::MolOps::Kekulize(*this->rdkit_molecule);
+    if(sanitize_after) {
+        RDKit::MolOps::Kekulize(*this->rdkit_molecule);
+    } else {
+        try {
+            RDKit::MolOps::Kekulize(*this->rdkit_molecule);
+        } catch(std::exception& e) {
+            g_warning("Could not kekulize molecule: %s", e.what());
+        }
+    }
 
     /// 2.1 Compute geometry
     auto geometry = this->compute_molecule_geometry();
 
     // 2.2 Build internal repr
-    this->build_internal_molecule_representation(geometry);    
+    this->build_internal_molecule_representation(geometry);
     this->cached_atom_coordinate_map = std::move(geometry);
 
     // 2.3 Reverse kekulization on the original molecule after lowering.
     if (sanitize_after) {
         RDKit::MolOps::sanitizeMol(*this->rdkit_molecule);
     }
+
+    // QED update
+    if (with_qed) {
+        if (sanitize_after) {
+            this->update_qed_info();
+        } else {
+            try {
+                this->update_qed_info();
+            } catch(std::exception& e) {
+                g_warning("Could not update QED info: %s", e.what());
+            }
+        }
+        
+    }
+    // Process problematic areas
+    this->process_problematic_areas(!sanitize_after);
 }
 
-void CanvasMolecule::highlight_atom(int atom_idx) {
+void CanvasMolecule::process_problematic_areas(bool allow_invalid_molecules) {
+    this->clear_highlights(HighlightType::Error);
+    if(!allow_invalid_molecules) {
+        return;
+    }
+    try {
+        auto problems = RDKit::MolOps::detectChemistryProblems(*this->rdkit_molecule);
+        for(const auto& eptr: problems) {
+            auto* raw_eptr = eptr.get();
+            auto* atom_sanitize_exception = dynamic_cast<RDKit::AtomSanitizeException*>(raw_eptr);
+            if(atom_sanitize_exception) {
+                
+                this->add_atom_highlight(atom_sanitize_exception->getAtomIdx(), HighlightType::Error);
+            }
+        }
+    } catch(std::exception& e) {
+        g_warning("Could not process problematic areas: %s", e.what());
+    }
+}
+
+void CanvasMolecule::update_qed_info() {
+
+    using QED = layla::RDKit::QED;
+
+    coot::layla::RDKit::QED::QEDproperties raw_props = QED::properties(*this->rdkit_molecule);
+    auto qed_score_and_ads = QED::qed(*this->rdkit_molecule, raw_props);
+    QEDInfo new_info;
+
+    new_info.alogp                             = raw_props.ALOGP;
+    new_info.molecular_polar_surface_area      = raw_props.PSA;
+    new_info.molecular_weight                  = raw_props.MW;
+    new_info.number_of_alerts                  = raw_props.ALERTS;
+    new_info.number_of_aromatic_rings          = raw_props.AROM;
+    new_info.number_of_hydrogen_bond_acceptors = raw_props.HBA;
+    new_info.number_of_hydrogen_bond_donors    = raw_props.HBD;
+    new_info.number_of_rotatable_bonds         = raw_props.ROTB;
+    new_info.qed_score = qed_score_and_ads.qed_score;
+    new_info.ads_mw = qed_score_and_ads.ads_mw;
+    new_info.ads_alogp = qed_score_and_ads.ads_alogp;
+    new_info.ads_hba = qed_score_and_ads.ads_hba;
+    new_info.ads_hbd = qed_score_and_ads.ads_hbd;
+    new_info.ads_psa = qed_score_and_ads.ads_psa;
+    new_info.ads_rotb = qed_score_and_ads.ads_rotb;
+    new_info.ads_arom = qed_score_and_ads.ads_arom;
+    new_info.ads_alert = qed_score_and_ads.ads_alerts;
+
+    g_debug("Updated QED: ALOGP=%f PSA=%f MW=%f ALERTS=%u AROM=%u HBA=%u HBD=%u ROTB=%u QED=%f",
+        new_info.alogp,
+        new_info.molecular_polar_surface_area,
+        new_info.molecular_weight,
+        new_info.number_of_alerts,
+        new_info.number_of_aromatic_rings,
+        new_info.number_of_hydrogen_bond_acceptors,
+        new_info.number_of_hydrogen_bond_donors,
+        new_info.number_of_rotatable_bonds,
+        new_info.qed_score
+    );
+
+    this->qed_info = new_info;
+}
+
+void CanvasMolecule::add_atom_highlight(int atom_idx, HighlightType htype) {
     //g_debug("Highlighted atom with idx=%i",atom_idx);
-    this->atoms.at(atom_idx).highlighted = true;
+    this->atoms.at(atom_idx).highlight |= static_cast<highlight_t>(htype);
 }
 
-void CanvasMolecule::highlight_bond(int atom_a, int atom_b) {
+void CanvasMolecule::add_bond_highlight(unsigned int atom_a, unsigned int atom_b, HighlightType htype) {
+
     //g_debug("Highlighted bond between atoms with indices %i and %i",atom_a,atom_b);
     auto bonds_for_atom_a = this->bond_map.find(atom_a);
-    if(bonds_for_atom_a == this->bond_map.end()) {
+    if (bonds_for_atom_a == this->bond_map.end()) {
         throw std::runtime_error("Bond doesn't exist");
     }
-    auto target = std::find_if(bonds_for_atom_a->second.begin(),bonds_for_atom_a->second.end(),[=](const auto& bond){
-        return bond->second_atom_idx == atom_b && bond->first_atom_idx == atom_a;
+    auto target = std::find_if(bonds_for_atom_a->second.begin(), bonds_for_atom_a->second.end(), [=](const auto& bond) {
+          return ((bond->second_atom_idx == atom_b) && (bond->first_atom_idx == atom_a))
+              || ((bond->second_atom_idx == atom_a) && (bond->first_atom_idx == atom_b));
     });
-    if (target == this->bonds.end()) {
+    if (target == bonds_for_atom_a->second.end()) {
         throw std::runtime_error("Bond doesn't exist");
     }
-    (*target)->highlighted = true;
+    (*target)->highlight |= static_cast<highlight_t>(htype);
 }
 
-void CanvasMolecule::clear_highlights() {
+void CanvasMolecule::clear_highlights(HighlightType htype) {
     for(auto& bond: this->bonds) {
-        bond->highlighted = false;
+        auto& highlight = bond->highlight;
+        highlight &= ~static_cast<highlight_t>(htype);
     }
     for(auto& atom: this->atoms) {
-        atom.highlighted = false;
+        auto& highlight = atom.highlight;
+        highlight &= ~static_cast<highlight_t>(htype);
+    }
+}
+
+void CanvasMolecule::add_highlight_to_all_bonds(HighlightType htype) {
+    for(auto& bond: this->bonds) {
+        auto& highlight = bond->highlight;
+        highlight |= static_cast<highlight_t>(htype);
     }
 }
 
@@ -972,7 +1110,7 @@ void CanvasMolecule::update_cached_atom_coordinate_map_after_atom_removal(unsign
         // greater than its' index have to be decremented.
         //
         // Now, I don't want to mess around with iterator invalidation.
-        // So the easiest course of action is to just copy away all 
+        // So the easiest course of action is to just copy away all
         // of the affected elements, change them, remove them from the original map
         // and put them back there again.
         std::vector<std::pair<int,RDGeom::Point2D>> altered_elements;
