@@ -59,7 +59,7 @@
 // #define STB_IMAGE_IMPLEMENTATION
 // #define STB_IMAGE_WRITE_IMPLEMENTATION
 
-#include "tiny_gltf.h"
+#include "coot-utils/tiny_gltf.h"
 
 
 // for the moment make the scales explict, when fixed make the scales default
@@ -93,7 +93,8 @@ TextureMesh::setup_camera_facing_quad(float scale_x, float scale_y, float offset
 }
 
 void
-TextureMesh::setup_tomo_quad(float scale_x, float scale_y, float x_offset, float y_offset, float z_pos) {
+TextureMesh::setup_tomo_quad(float scale_x, float scale_y, float x_offset, float y_offset, float z_pos,
+                             bool texture_x_y_swap_flag) {
 
   draw_this_mesh = true;
 
@@ -103,11 +104,19 @@ TextureMesh::setup_tomo_quad(float scale_x, float scale_y, float x_offset, float
    vertices.clear();
    triangles.clear();
 
-   // the indexing might well be wrong here
-   vertices.push_back(TextureMeshVertex(glm::vec3(x_offset,           y_offset,           z_pos), n, col, glm::vec2(0,0)));
-   vertices.push_back(TextureMeshVertex(glm::vec3(x_offset + scale_x, y_offset,           z_pos), n, col, glm::vec2(0,1)));
-   vertices.push_back(TextureMeshVertex(glm::vec3(x_offset + scale_x, y_offset + scale_y, z_pos), n, col, glm::vec2(1,1)));
-   vertices.push_back(TextureMeshVertex(glm::vec3(x_offset,           y_offset + scale_y, z_pos), n, col, glm::vec2(1,0)));
+   // the logic/indexing might well be wrong here - fix in future
+
+   if (texture_x_y_swap_flag) {
+      vertices.push_back(TextureMeshVertex(glm::vec3(x_offset,           y_offset + scale_y, z_pos), n, col, glm::vec2(0,1)));
+      vertices.push_back(TextureMeshVertex(glm::vec3(x_offset + scale_x, y_offset + scale_y, z_pos), n, col, glm::vec2(1,1)));
+      vertices.push_back(TextureMeshVertex(glm::vec3(x_offset + scale_x, y_offset, z_pos), n, col, glm::vec2(1,0)));
+      vertices.push_back(TextureMeshVertex(glm::vec3(x_offset,           y_offset, z_pos), n, col, glm::vec2(0,0)));
+   } else {
+      vertices.push_back(TextureMeshVertex(glm::vec3(x_offset,           y_offset,           z_pos), n, col, glm::vec2(0,0)));
+      vertices.push_back(TextureMeshVertex(glm::vec3(x_offset + scale_x, y_offset,           z_pos), n, col, glm::vec2(0,1)));
+      vertices.push_back(TextureMeshVertex(glm::vec3(x_offset + scale_x, y_offset + scale_y, z_pos), n, col, glm::vec2(1,1)));
+      vertices.push_back(TextureMeshVertex(glm::vec3(x_offset,           y_offset + scale_y, z_pos), n, col, glm::vec2(1,0)));
+   }
 
    triangles.push_back(g_triangle(0,1,2));
    triangles.push_back(g_triangle(2,3,0));
@@ -945,19 +954,39 @@ TextureMesh::import(const IndexedModel &ind_model, float scale) {
 
 
 // 20220129-PE why did I delete this for crows?
+// 20250310-PE Ah, possibly because it used glBufferData and instancing should
+// update using glBufferSubData() - which this function now does.
 void
 TextureMesh::update_instancing_buffer_data(const std::vector<glm::vec3> &positions) {
 
    if (vao == VAO_NOT_SET)
-      std::cout << "You forget to setup this TextureMesh in update_instancing_buffer_data() "
+      std::cout << "ERROR:: in update_instancing_buffer_data(): You forgot to setup this TextureMesh "
                 << name << std::endl;
+
+   GLenum err = glGetError();
+   if (err)
+      std::cout << "GL ERROR:: TextureMesh::update_instancing_buffers() --- start --- " << _(err) << std::endl;
    glBindVertexArray(vao);
-   glBindBuffer(GL_ARRAY_BUFFER, inst_positions_id);
+   err = glGetError();
+   if (err)
+      std::cout << "GL ERROR:: TextureMesh::setup_instancing_buffers() post binding vao " << _(err) << std::endl;
+
+   // glBindBuffer(GL_ARRAY_BUFFER, inst_positions_id);
+   // err = glGetError();
+   // if (err)
+   //    std::cout << "GL ERROR:: TextureMesh::setup_instancing_buffers() post bind buffer " << _(err) << std::endl;
+
    int n_positions = positions.size();
    n_instances = n_positions;
    if (n_positions > n_instances_allocated)
       n_positions = n_instances_allocated;
-  glBufferData(GL_ARRAY_BUFFER, n_positions * sizeof(glm::vec3), &(positions[0]), GL_STATIC_DRAW);
+
+   if (positions.empty()) return;
+
+   // glBufferData(GL_ARRAY_BUFFER, n_positions * sizeof(glm::vec3), &(positions[0]), GL_STATIC_DRAW);
+
+   glBindBuffer(GL_ARRAY_BUFFER, inst_positions_id); // 20250312-PE needed.
+   glBufferSubData(GL_ARRAY_BUFFER, 0, n_positions * sizeof(glm::vec3), &(positions[0]));
 
 }
 
@@ -969,6 +998,13 @@ TextureMesh::update_instancing_buffer_data_for_happy_faces(const std::vector<glm
                                            unsigned int draw_count_max,
                                            const glm::vec3 &screen_y_uv) {
 
+   if (vao == VAO_NOT_SET)
+      std::cout << "ERROR:: in update_instancing_buffer_data_for_happy_faces(): You forgot to setup this TextureMesh "
+                << name << std::endl;
+
+   GLenum err = glGetError();
+   if (err)
+      std::cout << "GL ERROR:: TextureMesh::update_instancing_buffer_data_for_happy_faces() --- start --- " << _(err) << std::endl;
    glBindVertexArray(vao);
    draw_count = draw_count_in; // do I need this to be a class data item?
    std::vector<glm::vec3> positions(positions_in);
