@@ -127,6 +127,8 @@ graphics_info_t::draw_generic_objects(unsigned int pass_type) {
 
    // This is the function that draws clash spike capped cylinders
 
+   // std::cout << "draw_generic_objects() " << pass_type << std::endl;
+
    if (! generic_display_objects.empty()) {
 
       glm::vec3 eye_position = get_world_space_eye_position();
@@ -134,9 +136,11 @@ graphics_info_t::draw_generic_objects(unsigned int pass_type) {
       glm::mat4 model_rotation = get_model_rotation();
       glm::vec4 bg_col(background_colour, 1.0);
       Shader &shader = shader_for_moleculestotriangles;
-      shader = shader_for_meshes_with_shadows;
+      // 20250125-PE shader = shader_for_meshes_with_shadows; what an idiot
 
       glDisable(GL_BLEND);
+      // consider
+      // glEnable(GL_BLEND);
 
       bool do_depth_fog = true;
       for (unsigned int i=0; i<generic_display_objects.size(); i++) {
