@@ -723,11 +723,10 @@ on_smiles_to_simple_3d_ok_button_clicked(GtkButton       *button,
 
 extern "C" G_MODULE_EXPORT
 void
-on_diff_map_peaks_close_button_clicked
-                                        (GtkButton       *button,
-                                        gpointer         user_data)
-{
-   GtkWidget *vbox = widget_from_builder("dialog-vbox78");
+on_diff_map_peaks_close_button_clicked(GtkButton       *button,
+                                       gpointer         user_data) {
+
+   GtkWidget *vbox = widget_from_builder("dialog-vbox78"); // change this one day.
    clear_diff_map_peaks();
    gtk_widget_set_visible(vbox, FALSE);
    graphics_info_t::hide_vertical_validation_frame_if_appropriate();
