@@ -46,8 +46,10 @@
 #include "read-phs.h"
 #include "gtk-manual.h"
 #include "c-interface-refine.h"
+#include "cc-interface.hh" // 20250310-PE for set_use_primary_mouse_button_for_view_rotation()
 #include "utils/coot-utils.hh"
 
+#include "graphics-info.h" // for grab focus
 #include "widget-from-builder.hh"
 
 // this from callbacks.h (which I don't want to include here)
@@ -139,34 +141,29 @@ on_preferences_reset_button_clicked    (GtkButton       *button,
 }
 
 extern "C" G_MODULE_EXPORT
-void
-on_preferences_destroy                 (GtkWidget       *object,
-                                        gpointer         user_data)
-{
-  clear_preferences();
+gboolean
+on_preferences_close_request(GtkWidget       *dialog,
+                             gpointer         user_data) {
+   gtk_widget_set_visible(dialog, FALSE);
+   graphics_info_t g;
+   g.graphics_grab_focus();
+   return TRUE; // has been handled - no need to find another handler.
 }
 
+void set_use_trackpad(short int state); // or #include cc-interface.hh
+
 extern "C" G_MODULE_EXPORT
 void
-on_preferences_geometry_cis_peptide_bad_yes_radiobutton_toggled
-                                        (GtkCheckButton *checkbutton,
-                                         gpointer         user_data) {
-
+on_preferences_view_rotation_left_mouse_checkbutton_toggled(GtkCheckButton *checkbutton,
+                                                            gpointer         user_data) {
    if (gtk_check_button_get_active(checkbutton)) {
-      preferences_internal_change_value_int(PREFERENCES_MARK_CIS_BAD, 1);
-      set_mark_cis_peptides_as_bad(1);
-   }
-}
-
-extern "C" G_MODULE_EXPORT
-void
-on_preferences_geometry_cis_peptide_bad_no_radiobutton_toggled
-                                        (GtkCheckButton *checkbutton,
-                                        gpointer         user_data) {
-
-   if (gtk_check_button_get_active(checkbutton)) {
-      preferences_internal_change_value_int(PREFERENCES_MARK_CIS_BAD, 0);
-      set_mark_cis_peptides_as_bad(0);
+      preferences_internal_change_value_int(PREFERENCES_VIEW_ROTATION_MOUSE_BUTTON, 1);
+      // set_use_trackpad(1);
+      set_use_primary_mouse_button_for_view_rotation(1);
+   } else {
+      preferences_internal_change_value_int(PREFERENCES_VIEW_ROTATION_MOUSE_BUTTON, 0);
+      // set_use_trackpad(0);
+      set_use_primary_mouse_button_for_view_rotation(0);
    }
 }
 
@@ -472,61 +469,50 @@ on_preferences_map_dynamic_sampling_checkbutton_toggled
 
 extern "C" G_MODULE_EXPORT
 void
-on_preferences_map_dynamic_size_checkbutton_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  if (gtk_toggle_button_get_active(togglebutton)) {
-    preferences_internal_change_value_int(PREFERENCES_DYNAMIC_MAP_SIZE_DISPLAY, 1);
-    set_dynamic_map_size_display_on();
-  } else {
-    preferences_internal_change_value_int(PREFERENCES_DYNAMIC_MAP_SIZE_DISPLAY, 0);
-    set_dynamic_map_size_display_off();
-  }
-
+on_preferences_map_dynamic_size_checkbutton_toggled(GtkCheckButton *checkbutton,
+                                                    gpointer         user_data) {
+   if (gtk_check_button_get_active(checkbutton)) {
+      preferences_internal_change_value_int(PREFERENCES_DYNAMIC_MAP_SIZE_DISPLAY, 1);
+      set_dynamic_map_size_display_on();
+   } else {
+      preferences_internal_change_value_int(PREFERENCES_DYNAMIC_MAP_SIZE_DISPLAY, 0);
+      set_dynamic_map_size_display_off();
+   }
 }
 
 
 extern "C" G_MODULE_EXPORT
 void
-on_preferences_diff_map_colours_coot_radiobutton_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  if (gtk_toggle_button_get_active(togglebutton)) {
-    preferences_internal_change_value_int(PREFERENCES_SWAP_DIFF_MAP_COLOURS, 0);
-    set_swap_difference_map_colours(0);
-  }
-
+on_preferences_diff_map_colours_coot_radiobutton_toggled(GtkCheckButton *checkbutton,
+                                                         gpointer        user_data) {
+   if (gtk_check_button_get_active(checkbutton)) {
+      preferences_internal_change_value_int(PREFERENCES_SWAP_DIFF_MAP_COLOURS, 0);
+      set_swap_difference_map_colours(0);
+   }
 }
 
 
 extern "C" G_MODULE_EXPORT
 void
-on_preferences_diff_map_colours_o_radiobutton_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  if (gtk_toggle_button_get_active(togglebutton)) {
-    preferences_internal_change_value_int(PREFERENCES_SWAP_DIFF_MAP_COLOURS, 1);
-    set_swap_difference_map_colours(1);
-  }
-
+on_preferences_diff_map_colours_o_radiobutton_toggled(GtkCheckButton *checkbutton,
+                                                      gpointer         user_data) {
+   if (gtk_check_button_get_active(checkbutton)) {
+      preferences_internal_change_value_int(PREFERENCES_SWAP_DIFF_MAP_COLOURS, 1);
+      set_swap_difference_map_colours(1);
+   }
 }
 
 
 extern "C" G_MODULE_EXPORT
 void
-on_preferences_map_colours_hscale_value_changed
-                                        (GtkRange        *range,
-                                        gpointer         user_data)
-{
-  GtkAdjustment *adjustment;
-  float fvalue;
-  adjustment = gtk_range_get_adjustment(GTK_RANGE(range));
-  fvalue = gtk_adjustment_get_value(adjustment);
-  preferences_internal_change_value_float(PREFERENCES_MAP_COLOURS_MAP_ROTATION, fvalue);
-  set_colour_map_rotation_for_map(fvalue);
+on_preferences_map_colours_hscale_value_changed(GtkRange        *range,
+                                                gpointer         user_data) {
+   GtkAdjustment *adjustment;
+   float fvalue;
+   adjustment = gtk_range_get_adjustment(GTK_RANGE(range));
+   fvalue = gtk_adjustment_get_value(adjustment);
+   preferences_internal_change_value_float(PREFERENCES_MAP_COLOURS_MAP_ROTATION, fvalue);
+   set_colour_map_rotation_for_map(fvalue);
 }
 
 
@@ -537,7 +523,6 @@ on_preferences_smooth_scroll_on_radiobutton_toggled(GtkCheckButton *checkbutton,
 
    if (gtk_check_button_get_active(checkbutton)) {
       preferences_internal_change_value_int(PREFERENCES_SMOOTH_SCROLL, 1);
-      std::cout << "EPH smooth scroll 1 " << std::endl;
       set_smooth_scroll_flag(1);
    }
 }
@@ -550,7 +535,6 @@ on_preferences_smooth_scroll_off_radiobutton_toggled(GtkCheckButton *checkbutton
 
    if (gtk_check_button_get_active(checkbutton)) {
       preferences_internal_change_value_int(PREFERENCES_SMOOTH_SCROLL, 0);
-      std::cout << "EPH smooth scroll 0 " << std::endl;
       set_smooth_scroll_flag(0);
   }
 
@@ -646,7 +630,6 @@ on_preferences_map_drag_on_radiobutton_toggled(GtkCheckButton *checkbutton,
 
    if (gtk_check_button_get_active(checkbutton)) {
       preferences_internal_change_value_int(PREFERENCES_MAP_DRAG, 1);
-      std::cout << "set_active_map_drag_flag 1 " << std::endl;
       set_active_map_drag_flag(1);
    }
 
@@ -659,41 +642,12 @@ on_preferences_map_drag_off_radiobutton_toggled(GtkCheckButton *checkbutton,
                                                 gpointer         user_data) {
 
    if (gtk_check_button_get_active(checkbutton)) {
-    preferences_internal_change_value_int(PREFERENCES_MAP_DRAG, 0);
-    set_active_map_drag_flag(0);
-      std::cout << "set_active_map_drag_flag 0 " << std::endl;
-  }
-
+      preferences_internal_change_value_int(PREFERENCES_MAP_DRAG, 0);
+      set_active_map_drag_flag(0);
+   }
 }
 
 
-extern "C" G_MODULE_EXPORT
-void
-on_preferences_antialias_on_radiobutton_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  if (gtk_toggle_button_get_active(togglebutton)) {
-    preferences_internal_change_value_int(PREFERENCES_ANTIALIAS, 1);
-    set_do_anti_aliasing(1);
-  }
-
-}
-
-
-extern "C" G_MODULE_EXPORT
-void
-on_preferences_antialias_off_radiobutton_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  if (gtk_toggle_button_get_active(togglebutton)) {
-    preferences_internal_change_value_int(PREFERENCES_ANTIALIAS, 0);
-    set_do_anti_aliasing(0);
-  }
-
-}
-
 
 extern "C" G_MODULE_EXPORT
 void
@@ -704,7 +658,6 @@ on_preferences_hid_spherical_radiobutton_toggled(GtkCheckButton *checkbutton,
       preferences_internal_change_value_int(PREFERENCES_VT_SURFACE, 2);
       vt_surface(2);
    }
-
 }
 
 
@@ -1244,171 +1197,6 @@ on_preferences_bond_width_combobox_changed
 }
 
 
-extern "C" G_MODULE_EXPORT
-void
-on_preferences_model_toolbar_show_radiobutton_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  if (gtk_toggle_button_get_active(togglebutton)) {
-    show_modelling_toolbar();
-    preferences_internal_change_value_int(PREFERENCES_MODEL_TOOLBAR_SHOW, 1);
-  }
-
-}
-
-
-extern "C" G_MODULE_EXPORT
-void
-on_preferences_model_toolbar_right_radiobutton_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  if (gtk_toggle_button_get_active(togglebutton)) {
-    set_model_toolbar_docked_position(0);
-    preferences_internal_change_value_int(PREFERENCES_MODEL_TOOLBAR_POSITION, 0);
-  }
-
-}
-
-
-extern "C" G_MODULE_EXPORT
-void
-on_preferences_model_toolbar_left_radiobutton_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  if (gtk_toggle_button_get_active(togglebutton)) {
-    set_model_toolbar_docked_position(1);
-    preferences_internal_change_value_int(PREFERENCES_MODEL_TOOLBAR_POSITION, 1);
-  }
-
-}
-
-
-extern "C" G_MODULE_EXPORT
-void
-on_preferences_model_toolbar_top_radiobutton_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  if (gtk_toggle_button_get_active(togglebutton)) {
-    set_model_toolbar_docked_position(2);
-    preferences_internal_change_value_int(PREFERENCES_MODEL_TOOLBAR_POSITION, 2);
-  }
-}
-
-
-extern "C" G_MODULE_EXPORT
-void
-on_preferences_model_toolbar_bottom_radiobutton_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  if (gtk_toggle_button_get_active(togglebutton)) {
-    set_model_toolbar_docked_position(3);
-    preferences_internal_change_value_int(PREFERENCES_MODEL_TOOLBAR_POSITION, 3);
-  }
-}
-
-
-extern "C" G_MODULE_EXPORT
-void
-on_preferences_model_toolbar_hide_radiobutton_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  if (gtk_toggle_button_get_active(togglebutton)) {
-    hide_modelling_toolbar();
-    preferences_internal_change_value_int(PREFERENCES_MODEL_TOOLBAR_SHOW, 0);
-  }
-
-}
-
-
-extern "C" G_MODULE_EXPORT
-void
-on_preferences_model_toolbar_main_icons_radiobutton_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-
-}
-
-
-extern "C" G_MODULE_EXPORT
-void
-on_preferences_model_toolbar_all_icons_radiobutton_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-
-}
-
-
-extern "C" G_MODULE_EXPORT
-void
-on_preferences_model_toolbar_style_icons_radiobutton_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  if (gtk_toggle_button_get_active(togglebutton)) {
-    preferences_internal_change_value_int(PREFERENCES_MODEL_TOOLBAR_STYLE, 1);
-    set_model_toolbar_style(1);
-  }
-
-}
-
-
-extern "C" G_MODULE_EXPORT
-void
-on_preferences_model_toolbar_style_both_radiobutton_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  if (gtk_toggle_button_get_active(togglebutton)) {
-    preferences_internal_change_value_int(PREFERENCES_MODEL_TOOLBAR_STYLE, 2);
-    set_model_toolbar_style(2);
-  }
-
-}
-
-
-extern "C" G_MODULE_EXPORT
-void
-on_preferences_model_toolbar_style_text_radiobutton_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  if (gtk_toggle_button_get_active(togglebutton)) {
-    preferences_internal_change_value_int(PREFERENCES_MODEL_TOOLBAR_STYLE, 3);
-    set_model_toolbar_style(3);
-  }
-
-}
-
-
-extern "C" G_MODULE_EXPORT
-void
-on_preferences_model_toolbar_show_icon_all_button_clicked
-                                        (GtkButton       *button,
-                                        gpointer         user_data)
-{
-  show_model_toolbar_all_icons();
-
-}
-
-
-extern "C" G_MODULE_EXPORT
-void
-on_preferences_model_toolbar_show_icon_selection_button_clicked
-                                        (GtkButton       *button,
-                                        gpointer         user_data)
-{
-  show_model_toolbar_main_icons();
-
-}
-
 extern "C" G_MODULE_EXPORT
 void
 on_preferences_main_toolbar_show_radiobutton_toggled
@@ -1508,10 +1296,8 @@ on_preferences_main_toolbar_style_both_radiobutton_toggled
 
 extern "C" G_MODULE_EXPORT
 void
-on_preferences_main_toolbar_style_text_radiobutton_toggled
-                                        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
+on_preferences_main_toolbar_style_text_radiobutton_toggled(GtkToggleButton *togglebutton,
+                                                           gpointer         user_data) {
   if (gtk_toggle_button_get_active(togglebutton)) {
     preferences_internal_change_value_int(PREFERENCES_MAIN_TOOLBAR_STYLE, 3);
     set_main_toolbar_style(3);
@@ -1519,3 +1305,25 @@ on_preferences_main_toolbar_style_text_radiobutton_toggled
 
 }
 
+extern "C" G_MODULE_EXPORT
+void
+noughties_physics_switch_state_set(GtkSwitch *switch_widget,
+                                   gboolean   state,
+                                   gpointer   user_data) {
+
+   if (state)
+      set_show_unit_cells_all(1);
+  else
+     set_show_unit_cells_all(0);
+}
+
+extern "C" G_MODULE_EXPORT
+void
+on_noughties_physics_checkbutton_toggled(GtkCheckButton *toggletoolbutton,
+                                         gpointer         user_data) {
+
+   if (gtk_check_button_get_active(toggletoolbutton))
+      set_refine_use_noughties_physics(1);
+   else
+      set_refine_use_noughties_physics(0);
+}
