@@ -29,6 +29,7 @@
 #endif
 
 #include <random>
+#include <iomanip>
 
 #define GLM_ENABLE_EXPERIMENTAL
 // #include <glm/ext.hpp>
@@ -39,56 +40,62 @@
 
 std::vector<std::reference_wrapper<Shader> > get_shader_refs() {
 
-   return std::vector<std::reference_wrapper<Shader> > { graphics_info_t::shader_for_maps,
-                                                        graphics_info_t::shader_for_map_caps,
-                                                        graphics_info_t::shader_for_models,
-                                                        graphics_info_t::shader_for_outline_of_active_residue,
-                                                        graphics_info_t::shader_for_model_as_meshes,
-                                                        graphics_info_t::shader_for_symmetry_atoms_bond_lines,
-                                                        graphics_info_t::shader_for_central_cube,
-                                                        graphics_info_t::shader_for_origin_cube,
-                                                        graphics_info_t::shader_for_hud_text,
-                                                        graphics_info_t::shader_for_hud_geometry_bars,
-                                                        graphics_info_t::shader_for_hud_geometry_labels,
-                                                        graphics_info_t::shader_for_hud_geometry_tooltip_text,
-                                                        graphics_info_t::shader_for_hud_buttons,
-                                                        graphics_info_t::shader_for_hud_image_texture,
-                                                        graphics_info_t::shader_for_atom_labels,
-                                                        graphics_info_t::shader_for_moleculestotriangles,
-                                                        graphics_info_t::shader_for_hud_lines,
-                                                        graphics_info_t::shader_for_lines,
-                                                        graphics_info_t::shader_for_lines_pulse,
-                                                        graphics_info_t::shader_for_rama_balls,
-                                                        graphics_info_t::shader_for_particles,
-                                                        graphics_info_t::shader_for_instanced_objects,
-                                                        graphics_info_t::shader_for_extra_distance_restraints,
-                                                        graphics_info_t::shader_for_happy_face_residue_markers,
-                                                        graphics_info_t::shader_for_happy_face_residue_markers_for_ssao,
-                                                        graphics_info_t::shader_for_rama_plot_phi_phis_markers,
-                                                        graphics_info_t::shader_for_rama_plot_axes_and_ticks,
-                                                        graphics_info_t::shader_for_ligand_view,
-                                                        graphics_info_t::shader_for_x_blur,
-                                                        graphics_info_t::shader_for_y_blur,
-                                                        graphics_info_t::shader_for_dof_blur_by_texture_combination,
-                                                        graphics_info_t::shader_for_texture_meshes,
-                                                        graphics_info_t::shader_for_meshes,
-                                                        graphics_info_t::shader_for_background_image,
-                                                        graphics_info_t::shader_for_meshes_with_shadows,
-                                                        graphics_info_t::shader_for_meshes_shadow_map,
-                                                        graphics_info_t::shader_for_instanced_meshes_shadow_map,
-                                                        graphics_info_t::shader_for_meshes_for_ssao,
-                                                        graphics_info_t::shader_for_instanced_meshes_with_shadows,
-                                                        graphics_info_t::shader_for_tmeshes_for_ssao,
-                                                        graphics_info_t::shader_for_tmeshes_with_shadows,
-                                                        graphics_info_t::shader_for_texture_meshes_shadow_map,
-                                                        graphics_info_t::shader_for_rotation_centre_cross_hairs_for_ssao,
-                                                        graphics_info_t::shader_for_tmeshes,
-                                                        graphics_info_t::shader_for_tmeshes_for_ssao,
-                                                        graphics_info_t::shader_for_shadow_map_image_texture_mesh,
-                                                        graphics_info_t::shader_for_effects,
-                                                        graphics_info_t::shaderGeometryPass,
-                                                        graphics_info_t::shaderSSAO,
-                                                        graphics_info_t::shaderSSAOBlur};
+   std::vector<std::reference_wrapper<Shader> > rs = { graphics_info_t::shader_for_maps,
+                                                       graphics_info_t::shader_for_map_caps,
+                                                       graphics_info_t::shader_for_models,
+                                                       graphics_info_t::shader_for_outline_of_active_residue,
+                                                       graphics_info_t::shader_for_model_as_meshes,
+                                                       graphics_info_t::shader_for_symmetry_atoms_bond_lines,
+                                                       graphics_info_t::shader_for_central_cube,
+                                                       graphics_info_t::shader_for_origin_cube,
+                                                       graphics_info_t::shader_for_hud_text,
+                                                       graphics_info_t::shader_for_hud_geometry_bars,
+                                                       graphics_info_t::shader_for_hud_geometry_labels,
+                                                       graphics_info_t::shader_for_hud_geometry_tooltip_text,
+                                                       graphics_info_t::shader_for_hud_buttons,
+                                                       graphics_info_t::shader_for_hud_image_texture,
+                                                       graphics_info_t::shader_for_atom_labels,
+                                                       graphics_info_t::shader_for_moleculestotriangles,
+                                                       graphics_info_t::shader_for_hud_lines,
+                                                       graphics_info_t::shader_for_lines,
+                                                       graphics_info_t::shader_for_lines_pulse,
+                                                       graphics_info_t::shader_for_rama_balls,
+                                                       graphics_info_t::shader_for_particles,
+                                                       graphics_info_t::shader_for_instanced_objects,
+                                                       graphics_info_t::shader_for_extra_distance_restraints,
+                                                       graphics_info_t::shader_for_happy_face_residue_markers,
+                                                       graphics_info_t::shader_for_rama_plot_phi_phis_markers,
+                                                       graphics_info_t::shader_for_rama_plot_axes_and_ticks,
+                                                       graphics_info_t::shader_for_ligand_view,
+                                                       graphics_info_t::shader_for_texture_meshes,
+                                                       graphics_info_t::shader_for_meshes,
+                                                       graphics_info_t::shader_for_background_image,
+                                                       graphics_info_t::shader_for_tmeshes};
+
+   if (! graphics_info_t::graphics_is_gl_es) {
+      rs.push_back(graphics_info_t::shader_for_effects);
+      rs.push_back(graphics_info_t::shader_for_dof_blur_by_texture_combination);
+      rs.push_back(graphics_info_t::shader_for_meshes_with_shadows);
+      rs.push_back(graphics_info_t::shader_for_meshes_shadow_map);
+      rs.push_back(graphics_info_t::shader_for_instanced_meshes_shadow_map);
+      rs.push_back(graphics_info_t::shader_for_meshes_for_ssao);
+      rs.push_back(graphics_info_t::shader_for_instanced_meshes_for_ssao);
+      rs.push_back(graphics_info_t::shader_for_instanced_meshes_with_shadows);
+      rs.push_back(graphics_info_t::shader_for_tmeshes_for_ssao);
+      rs.push_back(graphics_info_t::shader_for_tmeshes_with_shadows);
+      rs.push_back(graphics_info_t::shader_for_texture_meshes_shadow_map);
+      rs.push_back(graphics_info_t::shader_for_rotation_centre_cross_hairs_for_ssao);
+      rs.push_back(graphics_info_t::shader_for_tmeshes_for_ssao);
+      rs.push_back(graphics_info_t::shader_for_shadow_map_image_texture_mesh);
+      rs.push_back(graphics_info_t::shaderGeometryPass);
+      rs.push_back(graphics_info_t::shader_for_happy_face_residue_markers_for_ssao);
+      rs.push_back(graphics_info_t::shader_for_x_blur);
+      rs.push_back(graphics_info_t::shader_for_y_blur);
+      rs.push_back(graphics_info_t::shaderSSAO);
+      rs.push_back(graphics_info_t::shaderSSAOBlur);
+   }
+
+   return rs;
 }
 
 
@@ -115,61 +122,7 @@ graphics_info_t::init_shader(const std::string &shader_file_name) {
 bool
 graphics_info_t::init_shaders() {
 
-   std::vector<std::reference_wrapper<Shader> > shaders = {shader_for_maps,
-                                                           shader_for_map_caps,
-                                                           shader_for_models,
-                                                           shader_for_outline_of_active_residue,
-                                                           shader_for_model_as_meshes,
-                                                           shader_for_symmetry_atoms_bond_lines,
-                                                           shader_for_central_cube,
-                                                           shader_for_origin_cube,
-                                                           shader_for_hud_text,
-                                                           shader_for_hud_geometry_bars,
-                                                           shader_for_hud_geometry_labels,
-                                                           shader_for_hud_geometry_tooltip_text,
-                                                           shader_for_hud_buttons,
-                                                           shader_for_hud_image_texture,
-                                                           shader_for_atom_labels,
-                                                           shader_for_moleculestotriangles,
-                                                           shader_for_hud_lines,
-                                                           shader_for_lines,
-                                                           shader_for_lines_pulse,
-                                                           shader_for_rama_balls,
-                                                           shader_for_particles,
-                                                           shader_for_instanced_objects,
-                                                           shader_for_extra_distance_restraints,
-                                                           shader_for_happy_face_residue_markers,
-                                                           shader_for_happy_face_residue_markers_for_ssao,
-                                                           shader_for_rama_plot_phi_phis_markers,
-                                                           shader_for_rama_plot_axes_and_ticks,
-                                                           shader_for_ligand_view,
-                                                           shader_for_x_blur,
-                                                           shader_for_y_blur,
-                                                           shader_for_dof_blur_by_texture_combination,
-                                                           shader_for_texture_meshes,
-                                                           shader_for_meshes,
-                                                           shader_for_background_image,
-                                                           // camera_facing_quad_shader,
-
-                                                           // from crows
-                                                           shader_for_meshes_with_shadows,
-                                                           shader_for_meshes_shadow_map,
-                                                           shader_for_instanced_meshes_shadow_map,
-                                                           shader_for_meshes_for_ssao,
-                                                           shader_for_instanced_meshes_for_ssao,
-                                                           shader_for_instanced_meshes_with_shadows,
-                                                           shader_for_tmeshes_for_ssao,
-                                                           shader_for_tmeshes_with_shadows,
-                                                           shader_for_texture_meshes_shadow_map,
-                                                           shader_for_rotation_centre_cross_hairs_for_ssao,
-                                                           shader_for_tmeshes,
-                                                           shader_for_tmeshes_for_ssao,
-                                                           shader_for_shadow_map_image_texture_mesh,
-                                                           shader_for_effects,
-                                                           shaderGeometryPass,
-                                                           shaderSSAO,
-                                                           shaderSSAOBlur
-   };
+   std::vector<std::reference_wrapper<Shader> > shaders = get_shader_refs();
 
    bool status = true;  // success
 
@@ -181,22 +134,7 @@ graphics_info_t::init_shaders() {
    for (it=shaders.begin(); it!=shaders.end(); ++it)
       it->get().set_default_directory(d);
 
-   // crows
-   shader_for_meshes_with_shadows.init("meshes-with-shadows.shader",                Shader::Entity_t::MAP);
-   shader_for_meshes_shadow_map.init("meshes-for-shadow-map.shader",                Shader::Entity_t::MAP);
-   shader_for_instanced_meshes_shadow_map.init("instanced-meshes-for-shadow-map.shader", Shader::Entity_t::MAP);
-   shader_for_meshes_for_ssao.init("meshes-for-ssao.shader",                        Shader::Entity_t::MAP);
-   shader_for_instanced_meshes_for_ssao.init("instanced-meshes-for-ssao.shader",    Shader::Entity_t::MAP);
-   shader_for_tmeshes_for_ssao.init("texture-meshes-for-ssao.shader",               Shader::Entity_t::MAP);
-   shader_for_tmeshes_with_shadows.init("texture-meshes-with-shadows.shader",       Shader::Entity_t::MAP);
-   shader_for_texture_meshes_shadow_map.init("texture-meshes-shadow-map.shader",    Shader::Entity_t::MAP);
    shader_for_tmeshes.init("texture-meshes.shader",                                 Shader::Entity_t::MAP);  // Hmm! where is this used? (duplicate)
-   shader_for_shadow_map_image_texture_mesh.init("shadow-map-image-texture.shader", Shader::Entity_t::MAP);
-   shaderGeometryPass.init("9.ssao_geometry.shader", Shader::Entity_t::NONE);
-   shaderSSAO.init(        "9.ssao.shader",          Shader::Entity_t::NONE);
-   shaderSSAOBlur.init(    "9.ssao_blur.shader",     Shader::Entity_t::NONE);
-   shader_for_instanced_meshes_with_shadows.init("instanced-meshes-with-shadows.shader", Shader::Entity_t::MAP);
-
    shader_for_outline_of_active_residue.init("outline-of-active-residue.shader", Shader::Entity_t::MODEL);
    shader_for_maps.init("map.shader", Shader::Entity_t::MAP);
    shader_for_map_caps.init("draw-map-cap.shader", Shader::Entity_t::MAP);
@@ -217,7 +155,6 @@ graphics_info_t::init_shaders() {
    shader_for_extra_distance_restraints.init("extra-distance-restraints.shader", Shader::Entity_t::INSTANCED_DISPLAY_OBJECT);
    shader_for_hud_geometry_tooltip_text.init("hud-geometry-tooltip-text.shader", Shader::Entity_t::HUD_TEXT);
    shader_for_happy_face_residue_markers.init("residue-markers.shader", Shader::Entity_t::GENERIC_DISPLAY_OBJECT);
-   shader_for_happy_face_residue_markers_for_ssao.init("residue-markers-for-ssao.shader", Shader::Entity_t::GENERIC_DISPLAY_OBJECT);
    shader_for_ligand_view.init("ligand-view.shader", Shader::Entity_t::NONE);
    shader_for_model_as_meshes.init("model-as-mesh.shader", Shader::Entity_t::MODEL);
    shader_for_symmetry_atoms_bond_lines.init("symmetry-atoms-lines.shader", Shader::Entity_t::MAP);
@@ -228,19 +165,39 @@ graphics_info_t::init_shaders() {
    shader_for_background_image.init("background-image.shader", Shader::Entity_t::NONE);
    shader_for_meshes.init("meshes.shader", Shader::Entity_t::MAP); // 20220208-PE temporay while crow code is merged.
    shader_for_texture_meshes.init("texture-meshes.shader", Shader::Entity_t::MAP);
-   shader_for_effects.init("effects.shader", Shader::Entity_t::NONE);
 
-   // testing image textures
-   // camera_facing_quad_shader.init("camera-facing-quad-shader-for-testing.shader", Shader::Entity_t::MODEL);
+   if (graphics_is_gl_es) {
+   } else {
+      // crows
+      shader_for_happy_face_residue_markers_for_ssao.init("residue-markers-for-ssao.shader", Shader::Entity_t::GENERIC_DISPLAY_OBJECT);
+      shader_for_meshes_with_shadows.init("meshes-with-shadows.shader",                Shader::Entity_t::MAP);
+      shader_for_meshes_shadow_map.init("meshes-for-shadow-map.shader",                Shader::Entity_t::MAP);
+      shader_for_instanced_meshes_shadow_map.init("instanced-meshes-for-shadow-map.shader", Shader::Entity_t::MAP);
+      shader_for_meshes_for_ssao.init("meshes-for-ssao.shader",                        Shader::Entity_t::MAP);
+      shader_for_instanced_meshes_for_ssao.init("instanced-meshes-for-ssao.shader",    Shader::Entity_t::MAP);
+      shader_for_tmeshes_for_ssao.init("texture-meshes-for-ssao.shader",               Shader::Entity_t::MAP);
+      shader_for_tmeshes_with_shadows.init("texture-meshes-with-shadows.shader",       Shader::Entity_t::MAP);
+      shader_for_texture_meshes_shadow_map.init("texture-meshes-shadow-map.shader",    Shader::Entity_t::MAP);
+      shader_for_shadow_map_image_texture_mesh.init("shadow-map-image-texture.shader", Shader::Entity_t::MAP);
+      shaderGeometryPass.init("9.ssao_geometry.shader", Shader::Entity_t::NONE);
+      shaderSSAO.init(        "9.ssao.shader",          Shader::Entity_t::NONE);
+      shaderSSAOBlur.init(    "9.ssao_blur.shader",     Shader::Entity_t::NONE);
+      shader_for_instanced_meshes_with_shadows.init("instanced-meshes-with-shadows.shader", Shader::Entity_t::MAP);
 
-   // we use the above to make an image/texture in the framebuffer and use then
-   // shader_for_screen to convert that framebuffer to the screen buffer.
-   shader_for_x_blur.init("blur-x.shader", Shader::Entity_t::SCREEN);
-   shader_for_y_blur.init("blur-y.shader", Shader::Entity_t::SCREEN);
-   shader_for_dof_blur_by_texture_combination.init("depth-of-field.shader", Shader::Entity_t::SCREEN);
+      shader_for_effects.init("effects.shader", Shader::Entity_t::NONE);
 
-   // long name at the bottom
-   shader_for_rotation_centre_cross_hairs_for_ssao.init("rotation-centre-cross-hairs-for-ssao.shader", Shader::Entity_t::NONE);
+      // testing image textures
+      // camera_facing_quad_shader.init("camera-facing-quad-shader-for-testing.shader", Shader::Entity_t::MODEL);
+
+      // we use the above to make an image/texture in the framebuffer and use then
+      // shader_for_screen to convert that framebuffer to the screen buffer.
+      shader_for_x_blur.init("blur-x.shader", Shader::Entity_t::SCREEN);
+      shader_for_y_blur.init("blur-y.shader", Shader::Entity_t::SCREEN);
+      shader_for_dof_blur_by_texture_combination.init("depth-of-field.shader", Shader::Entity_t::SCREEN);
+
+      // long name at the bottom
+      shader_for_rotation_centre_cross_hairs_for_ssao.init("rotation-centre-cross-hairs-for-ssao.shader", Shader::Entity_t::NONE);
+   }
 
    for (it=shaders.begin(); it!=shaders.end(); ++it) {
       if (! it->get().get_success_status()) {
@@ -479,8 +436,11 @@ graphics_info_t::set_clipping_front(float v) {
    } else {
       clipping_front = v;
    }
-   std::cout << "DEBUG:: in set_clipping_front() now planes: front: " << clipping_front << " back: " << clipping_back
-             << " eye-position " << glm::to_string(eye_position) << std::endl;
+
+   // std::cout << "DEBUG:: in set_clipping_front() now planes: front: " << clipping_front
+   //           << " back: " << clipping_back
+   //           << " eye-position " << glm::to_string(eye_position) << std::endl;
+
    graphics_draw();
 }
 
@@ -647,29 +607,6 @@ graphics_info_t::set_view_quaternion(float i, float j, float k, float l) {
 }
 
 
-// //static
-// void
-// graphics_info_t::update_view_quaternion(int area_width, int area_height) {
-
-//    graphics_info_t g;
-//    float tbs = g.get_trackball_size();
-
-//    glm::quat tb_quat =
-//       g.trackball_to_quaternion((2.0*g.GetMouseBeginX() - area_width)/area_width,
-//                                 (area_height - 2.0*g.GetMouseBeginY())/area_height,
-//                                 (2.0*g.mouse_current_x - area_width)/area_width,
-//                                 (area_height - 2.0*g.mouse_current_y)/area_height,
-//                                 tbs);
-
-//    tb_quat = glm::conjugate(tb_quat); // hooray, no more "backwards" mouse motion
-//    glm::quat product = tb_quat * glm_quat;
-//    glm_quat = glm::normalize(product);
-
-// }
-
-
-#include <iomanip>
-
 void
 graphics_info_t::update_view_quaternion(int glarea_width, int glarea_height,
                                         double delta_x_drag, double delta_y_drag) {
@@ -697,10 +634,28 @@ graphics_info_t::update_view_quaternion(int glarea_width, int glarea_height,
 
    if (do_it) {
       glm::quat tb_quat = trackball_to_quaternion((2.0 * mouse_x - w)/w, (h - 2.0 * mouse_y)/h,
-                                                  (2.0 * current_mouse_x - w)/w, (h - 2.0 * current_mouse_y)/h, tbs);
+                                                  (2.0 * current_mouse_x - w)/w,
+                                                  (h - 2.0 * current_mouse_y)/h, tbs);
       tb_quat = glm::conjugate(tb_quat);
       auto prod = tb_quat * view_quaternion;
       view_quaternion = glm::normalize(prod);
+      // starting down z: view quaternion quat( 0.999986, { 0.003957,  0.002008, 0.002724})
+      // rotate above around screen z 180: view quaternion quat(0.000280, {0.003231, 0.006105, 0.999976})
+      // rotate above around screen y 180: view quaternion quat(-0.001343, {-0.999951, -0.006849, 0.006969})
+      // rotate above aournd screen z 180: view quaternion quat(0.008224, {-0.003389, 0.999938, 0.006631})
+
+      // down x: view quaternion quat(0.499870, {-0.501352, 0.497861, 0.500910})
+      // rotate screen z 180: view quaternion quat(-0.502290, {-0.498534, -0.498968, 0.500200})
+      // rotate screen y 180: view quaternion quat(0.497855, {0.505183, -0.492305, 0.504545})
+      // rotate screen z 180: view quaternion quat(0.503984, {-0.497813, -0.503251, -0.494895})
+
+      // down y: view quaternion quat(0.001501, {0.007544, -0.707126, -0.707045})
+      // rotate screen z: view quaternion quat(0.707459, {0.706673, 0.010734, 0.000401})
+      // rotate screen y: view quaternion quat(-0.005150, {0.003813, -0.709497, 0.704679})
+      // rotate screen z: view quaternion quat(-0.705722, {0.708459, 0.005714, -0.002985})
+
+      if (false)
+         std::cout << "view quaternion " << glm::to_string(view_quaternion) << std::endl;
    }
    mouse_x = current_mouse_x;
    mouse_y = current_mouse_y;
@@ -730,7 +685,8 @@ graphics_info_t::setup_cylinder_clashes(const coot::atom_overlaps_dots_container
    //
    graphics_info_t g;
    if (c.clashes.size() == 0) {
-      std::cout << "zero clashes" << std::endl;
+      if (false)
+         std::cout << "zero clashes" << std::endl;
       std::string clashes_name = get_clashes_object_name(imol);
       int clashes_obj_index = generic_object_index(clashes_name);
       if (clashes_obj_index == -1) {
@@ -965,6 +921,7 @@ graphics_info_t::coot_all_atom_contact_dots_instanced(mmdb::Manager *mol, int im
    };
 
    // more sensible
+
    coot::atom_overlaps_container_t overlaps(mol, graphics_info_t::Geom_p(), ignore_waters, 0.5, 0.25);
    bool do_vdw_surface = all_atom_contact_dots_do_vdw_surface; // static class variable
    c = overlaps.all_atom_contact_dots(contact_dots_density, do_vdw_surface);
