@@ -32,6 +32,7 @@
 #include <algorithm>
 
 #include <gtk/gtk.h>
+#include "utils/xdg-base.hh"
 #include "interface.h"
 #include "globjects.h" // for GRAPHICS_WINDOW_X_START_SIZE and Y
 #include "graphics-info.h"
@@ -77,9 +78,13 @@ graphics_info_t::save_state_file(const std::string &filename, short int il) {
       // python
       comment_str  = "# These commands are the saved state of coot.  You can evaluate them\n";
       comment_str += "# using \"Calculate->Run Script...\".\n\n";
-      comment_str += "import coot\n";           // these are not really comments, but we
-      comment_str += "import coot_gui\n";       // don't want to munge them.
-      comment_str += "import coot_utils\n"; // a better organization would be "import coot.utils"
+      // comment_str += "import coot\n";           // these are not really comments, but we
+      // comment_str += "import coot_gui\n";       // don't want to munge them.
+      // comment_str += "import coot_utils\n"; // a better organization would be "import coot.utils"
+
+      // 20241016-PE currently coot can't do coot_gui - so let's remove that import.
+      // There are no coot_gui functions in the state script AFAICS at the moment.
+      comment_str += "import coot\n";
    }
    commands.push_back(comment_str);
 
@@ -219,8 +224,8 @@ graphics_info_t::save_state_file(const std::string &filename, short int il) {
    // cif dictionary
    if (cif_dictionary_filename_vec->size() > 0) {
       for (unsigned int i=0; i<cif_dictionary_filename_vec->size(); i++) {
-         commands.push_back(state_command("coot", "read-cif-dictionary",
-                                          single_quote((*cif_dictionary_filename_vec)[i]), il));
+         auto fn = (*cif_dictionary_filename_vec)[i];
+         commands.push_back(state_command("coot", "read-cif-dictionary", fn, il));
       }
    }
 
@@ -236,6 +241,11 @@ graphics_info_t::save_state_file(const std::string &filename, short int il) {
    if (rotamer_search_mode == ROTAMERSEARCHLOWRES)
       commands.push_back(state_command("coot", "set-rotamer-search-mode", ROTAMERSEARCHLOWRES, il));
 
+   // Lighting
+   if (displayed_image_type == graphics_info_t::SHOW_AO_SCENE) // else SHOW_BASIC_SCENE
+      commands.push_back(state_command("coot", "set-use-fancy-lighting", 1, il));
+
+
    std::vector <std::string> command_strings;
 
    // because the goto_atom_molecule could be 11 with 8 closed
@@ -392,81 +402,48 @@ graphics_info_t::save_state_file(const std::string &filename, short int il) {
 
                if (molecules[i].Bonds_box_type() != coot::NORMAL_BONDS) {
                   if (molecules[i].Bonds_box_type() == coot::CA_BONDS) {
-                     active_strings.clear();
-                     display_strings.push_back("coot");
-                     active_strings.push_back("graphics-to-ca-representation");
-                     active_strings.push_back(int_to_string(molecule_count));
-                     commands.push_back(state_command(active_strings, il));
+                     std::string sc = state_command("coot", "graphics-to-ca-representation", molecule_count, il);
+                     commands.push_back(sc);
                   }
                   if (molecules[i].Bonds_box_type() == coot::COLOUR_BY_CHAIN_BONDS) {
-                     active_strings.clear();
-                     display_strings.push_back("coot");
-                     active_strings.push_back("set-colour-by-chain");
-                     active_strings.push_back(int_to_string(molecule_count));
-                     commands.push_back(state_command(active_strings, il));
+                     std::string sc = state_command("coot", "set-colour-by-chain", molecule_count, il);
+                     commands.push_back(sc);
                   }
                   if (molecules[i].Bonds_box_type() == coot::CA_BONDS_PLUS_LIGANDS) {
-                     active_strings.clear();
-                     display_strings.push_back("coot");
-                     active_strings.push_back("graphics-to-ca-plus-ligands-representation");
-                     active_strings.push_back(int_to_string(molecule_count));
-                     commands.push_back(state_command(active_strings, il));
+                     std::string sc = state_command("coot", "graphics-to-ca-plus-ligands-representation", molecule_count, il);
+                     commands.push_back(sc);
                   }
                   if (molecules[i].Bonds_box_type() == coot::CA_BONDS_PLUS_LIGANDS_AND_SIDECHAINS) {
-                     active_strings.clear();
-                     display_strings.push_back("coot");
-                     active_strings.push_back("graphics-to-ca-plus-ligands-and-sidechains-representation");
-                     active_strings.push_back(int_to_string(molecule_count));
-                     commands.push_back(state_command(active_strings, il));
+                     std::string sc = state_command("coot", "graphics-to-ca-plus-ligands-and-sidechains-representation", molecule_count, il);
+                     commands.push_back(sc);
                   }
                   if (molecules[i].Bonds_box_type() == coot::BONDS_NO_WATERS) {
-                     active_strings.clear();
-                     display_strings.push_back("coot");
-                     active_strings.push_back("graphics-to-bonds-no-waters-representation");
-                     active_strings.push_back(int_to_string(molecule_count));
-                     commands.push_back(state_command(active_strings, il));
+                     std::string sc = state_command("coot", "graphics-to-bonds-no-waters-representation", molecule_count, il);
+                     commands.push_back(sc);
                   }
                   if (molecules[i].Bonds_box_type() == coot::BONDS_SEC_STRUCT_COLOUR) {
-                     active_strings.clear();
-                     display_strings.push_back("coot");
-                     active_strings.push_back("graphics-to-sec-struct-bonds-representation");
-                     active_strings.push_back(int_to_string(molecule_count));
-                     commands.push_back(state_command(active_strings, il));
+                     std::string sc = state_command("coot", "graphics-to-sec-struct-bonds-representation", molecule_count, il);
+                     commands.push_back(sc);
                   }
                   if (molecules[i].Bonds_box_type() == coot::CA_BONDS_PLUS_LIGANDS_SEC_STRUCT_COLOUR) {
-                     active_strings.clear();
-                     display_strings.push_back("coot");
-                     active_strings.push_back("graphics-to-ca-plus-ligands-sec-struct-representation");
-                     active_strings.push_back(int_to_string(molecule_count));
-                     commands.push_back(state_command(active_strings, il));
+                     std::string sc = state_command("coot", "graphics-to-ca-plus-ligands-sec-struct-representation", molecule_count, il);
+                     commands.push_back(sc);
                   }
                   if (molecules[i].Bonds_box_type() == coot::COLOUR_BY_MOLECULE_BONDS) {
-                     active_strings.clear();
-                     display_strings.push_back("coot");
-                     active_strings.push_back("set-colour-by-molecule");
-                     active_strings.push_back(int_to_string(molecule_count));
-                     commands.push_back(state_command(active_strings, il));
+                     std::string sc = state_command("coot", "set-colour-by-molecule", molecule_count, il);
+                     commands.push_back(sc);
                   }
                   if (molecules[i].Bonds_box_type() == coot::COLOUR_BY_RAINBOW_BONDS) {
-                     active_strings.clear();
-                     display_strings.push_back("coot");
-                     active_strings.push_back("graphics-to-rainbow-representation");
-                     active_strings.push_back(int_to_string(molecule_count));
-                     commands.push_back(state_command(active_strings, il));
+                     std::string sc = state_command("coot", "graphics-to-rainbow-representation", molecule_count, il);
+                     commands.push_back(sc);
                   }
                   if (molecules[i].Bonds_box_type() == coot::COLOUR_BY_B_FACTOR_BONDS) {
-                     active_strings.clear();
-                     display_strings.push_back("coot");
-                     active_strings.push_back("graphics-to-b-factor-representation");
-                     active_strings.push_back(int_to_string(molecule_count));
-                     commands.push_back(state_command(active_strings, il));
+                     std::string sc = state_command("coot", "graphics-to-b-factor-representation", molecule_count, il);
+                     commands.push_back(sc);
                   }
                   if (molecules[i].Bonds_box_type() == coot::COLOUR_BY_OCCUPANCY_BONDS) {
-                     active_strings.clear();
-                     display_strings.push_back("coot");
-                     active_strings.push_back("graphics-to-occupancy-representation");
-                     active_strings.push_back(int_to_string(molecule_count));
-                     commands.push_back(state_command(active_strings, il));
+                     std::string sc = state_command("coot", "graphics-to-occupancy-representation", molecule_count, il);
+                     commands.push_back(sc);
                   }
                }
 
@@ -476,7 +453,7 @@ graphics_info_t::save_state_file(const std::string &filename, short int il) {
                   // First the "all" status for the additional representation of this molecule.
                   // Ooops. This is not (yet) a separate state.  Currently, pressing the button
                   // simply turns on (or off) all representations.  The display manager does not
-                  // save the state of this button (if it is off, closed and then openned then the
+                  // save the state of this button (if it is off, closed and then opened then the
                   // "all" button is not shown!)
 
                   //                   active_strings.clear();
@@ -651,17 +628,19 @@ graphics_info_t::save_state_file(const std::string &filename, short int il) {
       const coot::simple_distance_object_t &sdo = measure_distance_object_vec[idist];
       if (is_valid_model_molecule(sdo.imol_start)) {
          if (is_valid_model_molecule(sdo.imol_end)) {
-            std::vector<std::string> s(9);
-            s[0] = "add-geometry-distance";
-            s[1] = coot::util::int_to_string(sdo.imol_start);
-            s[2] = coot::util::float_to_string(sdo.start_pos.x());
-            s[3] = coot::util::float_to_string(sdo.start_pos.y());
-            s[4] = coot::util::float_to_string(sdo.start_pos.z());
-            s[5] = coot::util::int_to_string(sdo.imol_end);
-            s[6] = coot::util::float_to_string(sdo.end_pos.x());
-            s[7] = coot::util::float_to_string(sdo.end_pos.y());
-            s[8] = coot::util::float_to_string(sdo.end_pos.z());
-            commands.push_back(state_command(s,il));
+            std::vector<std::string> s(10);
+            s[0] = "coot";
+            s[1] = "add-geometry-distance";
+            s[2] = coot::util::int_to_string(sdo.imol_start);
+            s[3] = coot::util::float_to_string(sdo.start_pos.x());
+            s[4] = coot::util::float_to_string(sdo.start_pos.y());
+            s[5] = coot::util::float_to_string(sdo.start_pos.z());
+            s[6] = coot::util::int_to_string(sdo.imol_end);
+            s[7] = coot::util::float_to_string(sdo.end_pos.x());
+            s[8] = coot::util::float_to_string(sdo.end_pos.y());
+            s[9] = coot::util::float_to_string(sdo.end_pos.z());
+            std::string sc = state_command(s, il);
+            commands.push_back(sc);
          }
       }
    }
@@ -789,7 +768,7 @@ graphics_info_t::save_state_data_and_models(short int lang_flag) const {
    // add a hash at the start for python comments
    if (lang_flag == 2) {
       for (unsigned int i=0; i<v.size(); i++) {
-	      v[i] = "#" + v[i];
+         v[i] = "#" + v[i]; // this should be "# " to make a bone fide python comment
       }
    }
    return v;
@@ -842,42 +821,28 @@ graphics_info_t::save_state() {
 
    int r = 0;
 
-   // 20140410:
-   //
-   // This is a mess (but better than it was yesterday).  This needs a
-   // re-think about when the state file is written (and when
-   // run_state_file_status is set/unset).
+   xdg_t xdg;
+   std::filesystem::path path;
 
-   //    std::cout << "in save_state() run_state_file_status was " << run_state_file_status
-   // << std::endl;
-
-   if (run_state_file_status) {
-      short int il = coot::SCHEME_SCRIPT;
+   short int il = coot::SCHEME_SCRIPT;
 #ifdef USE_GUILE
-      if (1) { // was state_file_was_run_flag
-	 r = save_state_file(save_state_file_name, il);
-      } else {
-	 // std::cout << "state_file_was_run_flag was false" << std::endl;
-      }
+   path = xdg.get_state_home().append(save_state_file_name);
+
+   r = save_state_file(path.string(), il);
 #endif // USE_GUILE
 
-#ifdef USE_PYTHON
-      il = coot::PYTHON_SCRIPT;
-      // BL says:: grrr. This was a baddy using state_file_was_run_flag
-      // here. No idea what writing a state file has to do with running one.
-      if (1)
-	 r = save_state_file("0-coot.state.py", il);
-#endif // USE_PYTHON
-      return r;
-   }
-   return 0;
+   il = coot::PYTHON_SCRIPT;
+   // get_state_home() creates the directory if needed
+   path = xdg.get_state_home().append("0-coot.state.py");
+   r = save_state_file(path.string(), il);
+   return r;
 }
 
 std::string
 graphics_info_t::state_command(const std::string &name_space,
                                const std::string &str,
-			       int i1,
-			       short int state_lang) const {
+                               int i1,
+                               short int state_lang) const {
 
    std::vector<coot::command_arg_t> command_args;
    command_args.push_back(coot::command_arg_t(i1));
@@ -887,9 +852,9 @@ graphics_info_t::state_command(const std::string &name_space,
 std::string
 graphics_info_t::state_command(const std::string &name_space,
                                const std::string &str,
-			       int i1,
-			       int i2,
-			       short int state_lang) const {
+                               int i1,
+                               int i2,
+                               short int state_lang) const {
 
    std::vector<coot::command_arg_t> command_args;
    command_args.push_back(coot::command_arg_t(i1));
@@ -898,16 +863,17 @@ graphics_info_t::state_command(const std::string &name_space,
 }
 
 std::string
-graphics_info_t::state_command(const std::string &str,
-			       float f,
-			       short int state_lang) const {
+graphics_info_t::state_command(const std::string &name_space,
+                               const std::string &str,
+                               float f,
+                               short int state_lang) const {
 
-   std::string name_space = "coot"; // this is a guess/hack!
    std::vector<coot::command_arg_t> command_args;
    command_args.push_back(coot::command_arg_t(f));
    return state_command(name_space, str, command_args, state_lang);
 }
 
+// this needs to be cleaned up one day.
 std::string
 graphics_info_t::state_command(const std::string &str,
 			       float f,
@@ -975,11 +941,11 @@ graphics_info_t::state_command(const std::vector<std::string> &strs,
       }
 
       if (state_lang == coot::STATE_PYTHON) {
-	      if (strs.size() > 0) {
+         if (strs.size() > 0) {
             if (strs[0] == "coot") {
                command = "coot.";
                command += pythonize_command_name(strs[1]);
-	            command += "(";
+               command += "(";
                if (strs.size() > 2) {
                   // add args with commas after them
                   int n_strs_max = strs.size() -1;
@@ -989,8 +955,8 @@ graphics_info_t::state_command(const std::vector<std::string> &strs,
                   }
                }
             } else {
-   	         command = pythonize_command_name(strs[0]);
-	            command += "(";
+               command = pythonize_command_name(strs[0]);
+               command += "(";
                if (strs.size() > 2) {
                   // add args with commas after them
                   int n_strs_max = strs.size() -1;
@@ -1003,7 +969,7 @@ graphics_info_t::state_command(const std::vector<std::string> &strs,
             if (strs.size() > 1)
                command += strs.back();
             command +=  ")";
-	      }
+         }
       }
    }
    return command;
@@ -1017,7 +983,10 @@ graphics_info_t::state_command(const std::string &module, const std::string &fun
    std::string command;
 
    if (state_lang == coot::STATE_SCM) {
-      std::cout << "WARNING/ERROR:: missing new style state_command for scheme " << func_name << std::endl;
+
+      // 20240610-PE remove this for now
+      if (false)
+         std::cout << "WARNING/ERROR:: missing new style state_command for scheme " << func_name << std::endl;
    }
 
    if (state_lang == coot::STATE_PYTHON) {
@@ -1046,7 +1015,7 @@ graphics_info_t::state_command(const std::string &module, const std::string &fun
 //
 short int
 graphics_info_t::write_state(const std::vector<std::string> &commands,
-			     const std::string &filename) const {
+                             const std::string &filename) const {
 
    bool do_c_mode = false; // it's 2020 - Mac problems have gone away?
 
@@ -1067,7 +1036,7 @@ graphics_info_t::write_state(const std::vector<std::string> &commands,
 //
 short int
 graphics_info_t::write_state_fstream_mode(const std::vector<std::string> &commands,
-					  const std::string &filename) const {
+                                          const std::string &filename) const {
 
    short int istat = 1;
 
@@ -1076,7 +1045,7 @@ graphics_info_t::write_state_fstream_mode(const std::vector<std::string> &comman
 
    if (f) {
       for (unsigned int i=0; i<commands.size(); i++) {
-	 f << commands[i] << "\n";
+         f << commands[i] << "\n";
          // std::cout << "write_state_fstream_mode() " << commands[i] << std::endl;
       }
       f.flush();  // fixes valgrind problem?
@@ -1099,7 +1068,7 @@ graphics_info_t::write_state_fstream_mode(const std::vector<std::string> &comman
 
    } else {
       std::cout << "WARNING: couldn't write to state file " << filename
-		<< std::endl;
+                << std::endl;
       istat = 0;
    }
    return istat;
@@ -1191,6 +1160,7 @@ graphics_info_t::add_history_command(const std::vector<std::string> &command_str
 int
 graphics_info_t::save_history() const {
 
+   xdg_t xdg;
    int istate = 0;
    std::string history_file_name("0-coot-history");
    std::vector<std::vector<std::string> > raw_command_strings = history_list.history_list();
@@ -1198,15 +1168,17 @@ graphics_info_t::save_history() const {
    if (python_history) {
       for (unsigned int i=0; i<raw_command_strings.size(); i++)
 	 languaged_commands.push_back(pythonize_command_strings(raw_command_strings[i]));
-      std::string file = history_file_name + ".py";
-      istate =  write_state(languaged_commands, file);
+      std::string file_name = history_file_name + ".py";
+      std::string history_file_path = xdg.get_state_home().append(file_name).string();
+      istate =  write_state(languaged_commands, history_file_path);
    }
    if (guile_history) {
       languaged_commands.resize(0);
       for (unsigned int i=0; i<raw_command_strings.size(); i++)
 	 languaged_commands.push_back(schemize_command_strings(raw_command_strings[i]));
-      std::string file = history_file_name + ".scm";
-      istate =  write_state(languaged_commands, file);
+      std::string file_name = history_file_name + ".scm";
+      std::string history_file_path = xdg.get_state_home().append(file_name).string();
+      istate =  write_state(languaged_commands, history_file_path);
    }
    return istate;
 }
