@@ -28,12 +28,16 @@
 #include "Python.h"
 #endif
 
+#include "utils/xdg-base.hh"
 #include "graphics-info.h"
 #include "rotate-translate-modes.hh"
 #include "rotamer-search-modes.hh"
 
 #if !defined WINDOWS_MINGW
 
+int graphics_info_t::scale_down_graphics = 1;
+int graphics_info_t::scale_up_graphics = 1;
+
 #ifdef USE_GUILE
 #ifdef USE_GUILE_GTK
 bool graphics_info_t::prefer_python = 0; // prefer python scripts when
@@ -54,8 +58,6 @@ bool graphics_info_t::prefer_python = 0;
 bool graphics_info_t::scm_boot_guile_booted = false; // false until my_wrap_scm_boot_guile() has been run
 #endif
 
-
-
 #else // USE_GUILE test (no guile path)
 #ifdef USE_PYTHON
 bool graphics_info_t::prefer_python = 1; // Python, not guile
@@ -67,7 +69,13 @@ bool graphics_info_t::prefer_python = 0; // no GUILE or PYTHON
 bool graphics_info_t::prefer_python = 1; // Default: yes in Windows
 #endif // windows test
 
-bool graphics_info_t::using_trackpad = false;
+bool graphics_info_t::graphics_is_gl_es = false;
+
+// bool graphics_info_t::using_trackpad = false;
+bool graphics_info_t::use_primary_mouse_for_view_rotation_flag = false;
+
+
+logging graphics_info_t::log;
 
 bool graphics_info_t::use_gemmi = false;
 short int graphics_info_t::python_at_prompt_flag = 0;
@@ -89,12 +97,18 @@ ctpl::thread_pool graphics_info_t::static_thread_pool(coot::get_max_number_of_th
 #endif // HAVE_BOOST_BASED_THREAD_POOL_LIBRARY
 
 clipper::Xmap<float> *graphics_info_t::dummy_xmap = new clipper::Xmap<float>;
+std::vector<std::pair<std::string, clipper::Xmap<float> > > graphics_info_t::map_partition_results;
+int graphics_info_t::map_partition_results_state = 0; // inactive
+std::string graphics_info_t::map_partition_results_state_string; // "Done A Chain" etc.
+
 
 //WII
 #ifdef WII_INTERFACE_WIIUSE
 wiimote** graphics_info_t::wiimotes = NULL;
 #endif
 
+float graphics_info_t::view_rotation_per_pixel_scale_factor = 1.0;
+
 // Views
 float graphics_info_t::views_play_speed = 10.0;
 
@@ -188,6 +202,9 @@ float  graphics_info_t::rotation_centre_z = 0.0;
 // float  graphics_info_t::old_rotation_centre_y = 0.0;
 // float  graphics_info_t::old_rotation_centre_z = 0.0;
 coot::Cartesian graphics_info_t::old_rotation_centre(0,0,0);
+clipper::Coord_orth graphics_info_t::hole_start = clipper::Coord_orth(0,0,0);
+clipper::Coord_orth graphics_info_t::hole_end   = clipper::Coord_orth(0,0,0);
+
 float  graphics_info_t::zoom                = 100;
 int    graphics_info_t::smooth_scroll       =   1; // flag: default is ..
 int    graphics_info_t::smooth_scroll_n_steps =  20;
@@ -199,6 +216,7 @@ int    graphics_info_t::smooth_scroll_current_step = 0;
 coot::Cartesian graphics_info_t::smooth_scroll_delta;
 int    graphics_info_t::mouse_just_cliked     = 0;
 float  graphics_info_t::rotation_centre_cube_size = 0.1; // Angstroems
+glm::vec4 graphics_info_t::rotation_centre_cross_hairs_colour = glm::vec4(0.8, 0.8, 0.8, 1.0);
 short int graphics_info_t::quanta_like_zoom_flag = 0;
 int    graphics_info_t::go_to_ligand_animate_view_n_steps = 50;
 
@@ -289,6 +307,7 @@ double graphics_info_t::drag_begin_x = 0.0;
 double graphics_info_t::drag_begin_y = 0.0;
 double graphics_info_t::mouse_x = 0.0;
 double graphics_info_t::mouse_y = 0.0;
+double graphics_info_t::mouse_speed = 2.0;
 std::pair<double, double> graphics_info_t::mouse_previous_position = std::make_pair(0.0, 0.0);
 
 // residue reorientation on "space"
@@ -421,7 +440,11 @@ int graphics_info_t::undo_molecule = -1;
 
 // backup filenames
 bool graphics_info_t::unpathed_backup_file_names_flag = 0;
+#ifdef WINDOWS_MINGW
+bool graphics_info_t::decoloned_backup_file_names_flag = 1;
+#else
 bool graphics_info_t::decoloned_backup_file_names_flag = 0;
+#endif
 
 // backup compress files (default: compress)
 int graphics_info_t::backup_compress_files_flag = 1;
@@ -745,6 +768,8 @@ short int graphics_info_t::moving_atoms_asc_type = coot::NEW_COORDS_UNSET; // un
 int graphics_info_t::imol_moving_atoms = 0;
 coot::extra_restraints_representation_t graphics_info_t::moving_atoms_extra_restraints_representation;
 
+bool graphics_info_t::noughties_physics = false;
+
 bool graphics_info_t::draw_it_for_moving_atoms_restraints_graphics_object = false;
 bool graphics_info_t::draw_it_for_moving_atoms_restraints_graphics_object_user_control = false;
 int graphics_info_t::imol_refinement_map = -1; // magic initial value "None set"
@@ -896,7 +921,7 @@ float graphics_info_t::map_radius_slider_max = 50.0;
 short int graphics_info_t::rotate_colour_map_on_read_pdb_flag = 1; // do it.
 short int graphics_info_t::rotate_colour_map_on_read_pdb_c_only_flag = 1; // rotate Cs only by default
 float     graphics_info_t::rotate_colour_map_on_read_pdb = 21.0;  // degrees
-float     graphics_info_t::rotate_colour_map_for_map = 14.0;  // degrees
+float     graphics_info_t::rotate_colour_map_for_map = 31.0; // 20240907-PE 14.0;  // degrees
 
 float graphics_info_t::goodsell_chain_colour_wheel_rotation_step = 0.221;
 
@@ -1664,7 +1689,8 @@ bool           graphics_info_t::draw_background_image_flag = false; // uses "bac
 
 float graphics_info_t::pull_restraint_neighbour_displacement_max_radius = 0.0; // we don't see it initially.
 
-coot::command_history_t graphics_info_t::command_history;
+xdg_t xdg;
+coot::command_history_t graphics_info_t::command_history = coot::command_history_t(xdg);
 
 std::vector<Instanced_Markup_Mesh> graphics_info_t::instanced_meshes;
 
@@ -1821,3 +1847,6 @@ std::vector<coot::positron_metadata_t> graphics_info_t::positron_metadata;
 
 bool graphics_info_t::tomo_picker_flag = false;
 graphics_info_t::tomo_view_info_t graphics_info_t::tomo_view_info;
+
+std::pair<bool, std::string> graphics_info_t::servalcat_fofc    = std::pair<bool, std::string> (false, "");
+std::pair<bool, std::string> graphics_info_t::servalcat_refine  = std::pair<bool, std::string> (false, "");
